package org.processmining.acceptingpetrinet.filters;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.processmining.acceptingpetrinet.filters.impl.StartEndHideFilter;
import org.processmining.acceptingpetrinet.filters.impl.StartEndRemoveFilter;

public class StartEndFilterManager {

	private static StartEndFilterManager instance = null;
	private List<StartEndFilter> filters;
	private StartEndFilter defaultFilter;
	private boolean sorted;

	private StartEndFilterManager() {
		filters = new ArrayList<StartEndFilter>();
		defaultFilter = null;
		sorted = false;
		register(new StartEndHideFilter(), true);
		register(new StartEndRemoveFilter(), false);
	}

	public static StartEndFilterManager getInstance() {
		if (instance == null) {
			instance = new StartEndFilterManager();
		}
		return instance;
	}
	
	public void register(StartEndFilter filter, boolean isDefault) {
		filters.add(filter);
		if (isDefault) {
			defaultFilter = filter;
		}
		sorted = false;
	}

	public List<StartEndFilter> getFilters() {
		if (!sorted) {
			Collections.sort(filters, new Comparator<StartEndFilter>() {
				public int compare(StartEndFilter m1, StartEndFilter m2) {
					return m1.getName().compareTo(m2.getName());
				}
			});
			sorted = true;
		}
		return filters;
	}
	
	public boolean isDefault(StartEndFilter filter) {
		return filter == defaultFilter;
	}
	
	public StartEndFilter getFilter(String name) {
		if (name != null) {
			for (StartEndFilter filter : filters) {
				if (name.equals(filter.getName())) {
					return filter;
				}
			}
		}
		return defaultFilter;
	}

}
