package org.processmining.acceptingpetrinet.filters.impl;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.processmining.acceptingpetrinet.filters.StartEndFilter;
import org.processmining.acceptingpetrinet.filters.StartEndFilterParameters;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.models.impl.AcceptingPetriNetFactory;
import org.processmining.acceptingpetrinet.utils.PetriNetUtils;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;

public abstract class StartEndAbstractFilter implements StartEndFilter {

	public AcceptingPetriNet apply(PluginContext context, AcceptingPetriNet net, StartEndFilterParameters parameters) {
		AcceptingPetriNet reducedNet = AcceptingPetriNetFactory.createAcceptingPetriNet();

		Map<Place, Place> placeMap = new HashMap<Place, Place>();
		Map<Transition, Transition> transitionMap = new HashMap<Transition, Transition>();
		Petrinet pn = PetriNetUtils.cloneNet(net.getNet(), placeMap, transitionMap);
		Marking initialMarking = PetriNetUtils.apply(net.getInitialMarking(), placeMap);
		Set<Marking> finalMarkings = PetriNetUtils.apply(net.getFinalMarkings(), placeMap);

		apply(context, pn, initialMarking, finalMarkings, parameters);

		reducedNet.init(pn);
		reducedNet.setInitialMarking(initialMarking);
		reducedNet.setFinalMarkings(finalMarkings);
		return reducedNet;
	}

	public abstract void apply(PluginContext context, Petrinet net, Marking initialMarking, Set<Marking> finalMarkings,
			StartEndFilterParameters parameters);
}
