package org.processmining.acceptingpetrinet.plugins;

import java.util.Collection;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.acceptingpetrinet.algorithms.ConvertPetriNetToAcceptingPetriNetAlgorithm;
import org.processmining.acceptingpetrinet.connections.ConvertPetriNetToAcceptingPetriNetConnection;
import org.processmining.acceptingpetrinet.dialogs.ConvertPetriNetToAcceptingPetriNetDialog;
import org.processmining.acceptingpetrinet.help.ConvertPetriNetToAcceptingPetriNetHelp;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.parameters.ConvertPetriNetToAcceptingPetriNetParameters;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;

@Plugin(name = "Create Accepting Petri Net", parameterLabels = { "Petri Net", "Parameters" }, returnLabels = { "Accepting Petri Net" }, returnTypes = { AcceptingPetriNet.class }, help = ConvertPetriNetToAcceptingPetriNetHelp.TEXT)
public class ConvertPetriNetToAcceptingPetriNetPlugin extends ConvertPetriNetToAcceptingPetriNetAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Create Accepting Petri Net, UI", requiredParameterLabels = { 0 })
	public AcceptingPetriNet runUI(UIPluginContext context, Petrinet net) {
		ConvertPetriNetToAcceptingPetriNetParameters parameters = new ConvertPetriNetToAcceptingPetriNetParameters(net);
		ConvertPetriNetToAcceptingPetriNetDialog dialog = new ConvertPetriNetToAcceptingPetriNetDialog(context, net,
				parameters);
		int n = 0;
		String[] title = { "Configure creation (initial marking)", "Configure creation (final markings)" };
		InteractionResult result = InteractionResult.NEXT;
		while (result != InteractionResult.FINISHED) {
			result = context.showWizard(title[n], n == 0, n == 1, dialog.getPanel(n));
			if (result == InteractionResult.NEXT) {
				n++;
			} else if (result == InteractionResult.PREV) {
				n--;
			} else if (result != InteractionResult.FINISHED) {
				return null;
			}
		}
		return runConnection(context, net, parameters);
	}

	/**
	 * @deprecated Use runUI() instead.
	 */
	@Deprecated
	public AcceptingPetriNet convertUI(UIPluginContext context, Petrinet net) {
		return runUI(context, net);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Create Accepting Petri Net, Default", requiredParameterLabels = { 0 })
	public AcceptingPetriNet runDefault(PluginContext context, Petrinet net) {
		ConvertPetriNetToAcceptingPetriNetParameters parameters = new ConvertPetriNetToAcceptingPetriNetParameters(net);
		return runConnection(context, net, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public AcceptingPetriNet convertDefault(PluginContext context, Petrinet net) {
		return runDefault(context, net);
	}

	@PluginVariant(variantLabel = "Create Accepting Petri Net, Parameters", requiredParameterLabels = { 0, 1 })
	public AcceptingPetriNet run(PluginContext context, Petrinet net,
			ConvertPetriNetToAcceptingPetriNetParameters parameters) {
		return runConnection(context, net, parameters);
	}

	/**
	 * @deprecated Use run() instead.
	 */
	@Deprecated
	public AcceptingPetriNet convertParameters(PluginContext context, Petrinet net,
			ConvertPetriNetToAcceptingPetriNetParameters parameters) {
		return run(context, net, parameters);
	}

	private AcceptingPetriNet runConnection(PluginContext context, Petrinet net,
			ConvertPetriNetToAcceptingPetriNetParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<ConvertPetriNetToAcceptingPetriNetConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(
						ConvertPetriNetToAcceptingPetriNetConnection.class, context, net);
				for (ConvertPetriNetToAcceptingPetriNetConnection connection : connections) {
					if (connection.getObjectWithRole(ConvertPetriNetToAcceptingPetriNetConnection.NET).equals(net)
							&& connection.getParameters().equals(parameters)) {
						return connection.getObjectWithRole(ConvertPetriNetToAcceptingPetriNetConnection.ACCEPTINGNET);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		AcceptingPetriNet acceptingNet = apply(context, net, parameters);
		if (parameters.isTryConnections()) {
			context.getConnectionManager().addConnection(
					new ConvertPetriNetToAcceptingPetriNetConnection(net, acceptingNet, parameters));
		}
		return acceptingNet;
	}

}
