package org.processmining.acceptingpetrinet.plugins;

import java.util.Collection;

import org.processmining.acceptingpetrinet.algorithms.UnpackPetrinetAlgorithm;
import org.processmining.acceptingpetrinet.connections.UnpackPetrinetConnection;
import org.processmining.acceptingpetrinet.help.UnpackPetrinetHelp;
import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinet.parameters.UnpackPetrinetParameters;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;

@Plugin(name = "Unpack Accepting Petri net", level = PluginLevel.Regular, parameterLabels = { "Accepting Petri Net", "Parameters" }, returnLabels = { "Petri Net" }, returnTypes = { Petrinet.class }, help = UnpackPetrinetHelp.TEXT)
public class UnpackPetrinetPlugin extends UnpackPetrinetAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Unpack Accepting Petri net, Default", requiredParameterLabels = { 0 })
	public Petrinet runDefault(PluginContext context, AcceptingPetriNet apn) {
		UnpackPetrinetParameters parameters = new UnpackPetrinetParameters();
		return runConnection(context, apn, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public Petrinet unpackDefault(PluginContext context, AcceptingPetriNet apn) {
		return runDefault(context, apn);
	}

	@PluginVariant(variantLabel = "Unpack Accepting Petri net, Parameters", requiredParameterLabels = { 0, 1 })
	public Petrinet run(PluginContext context, AcceptingPetriNet apn, UnpackPetrinetParameters parameters) {
		return runConnection(context, apn, parameters);
	}

	/**
	 * @deprecated Use run() instead.
	 */
	@Deprecated
	public Petrinet reduceParameters(PluginContext context, AcceptingPetriNet apn, UnpackPetrinetParameters parameters) {
		return run(context, apn, parameters);
	}

	private Petrinet runConnection(PluginContext context, AcceptingPetriNet apn,
			UnpackPetrinetParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<UnpackPetrinetConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(UnpackPetrinetConnection.class, context,
						apn);
				for (UnpackPetrinetConnection connection : connections) {
					if (connection.getObjectWithRole(UnpackPetrinetConnection.APN).equals(apn)
							&& connection.getParameters().equals(parameters)) {
						return connection.getObjectWithRole(UnpackPetrinetConnection.NET);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		Petrinet net = apply(context, apn, parameters);
		if (parameters.isTryConnections()) {
			context.getConnectionManager().addConnection(new UnpackPetrinetConnection(apn, net, parameters));
		}
		return net;
	}

}
