package org.processmining.acceptingpetrinet.utils;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinet.impl.PetrinetFactory;
import org.processmining.models.semantics.petrinet.Marking;

public class PetriNetUtils {

	public static Petrinet cloneNet(Petrinet net, Map<Place, Place> placeMap, Map<Transition, Transition> transitionMap) {
		return cloneNet(net, placeMap, transitionMap, new HashMap<Arc, Arc>());
	}
	
	public static Petrinet cloneNet(Petrinet net, Map<Place, Place> placeMap, Map<Transition, Transition> transitionMap, Map<Arc, Arc> arcMap) {
		Petrinet clonedNet = PetrinetFactory.newPetrinet(net.getLabel());
		for (Place place : net.getPlaces()) {
			Place clonedPlace = clonedNet.addPlace(place.getLabel());
			placeMap.put(place, clonedPlace);
			for (String key: place.getAttributeMap().keySet()) {
				clonedPlace.getAttributeMap().put(key, place.getAttributeMap().get(key));
			}
		}
		for (Transition transition : net.getTransitions()) {
			Transition clonedTransition = clonedNet.addTransition(transition.getLabel());
			clonedTransition.setInvisible(transition.isInvisible());
			transitionMap.put(transition, clonedTransition);
			for (String key: transition.getAttributeMap().keySet()) {
				clonedTransition.getAttributeMap().put(key, transition.getAttributeMap().get(key));
			}
		}
		for (PetrinetEdge<?, ?> edge : net.getEdges()) {
			if (edge instanceof Arc) {
				Arc arc = (Arc) edge;
				if (placeMap.containsKey(arc.getSource())) {
					arcMap.put(arc, clonedNet
							.addArc(placeMap.get(arc.getSource()), transitionMap.get(arc.getTarget()), arc.getWeight()));
				} else {
					arcMap.put(arc, clonedNet
							.addArc(transitionMap.get(arc.getSource()), placeMap.get(arc.getTarget()), arc.getWeight()));
				}
				for (String key: arc.getAttributeMap().keySet()) {
					arcMap.get(arc).getAttributeMap().put(key, arc.getAttributeMap().get(key));
				}
			}
		}
		return clonedNet;
	}

	public static Marking apply(Marking marking, Map<Place, Place> map) {
		Marking appliedMarking = new Marking();
		for (Place place : marking.baseSet()) {
			appliedMarking.add(map.get(place), marking.occurrences(place));
		}
		return appliedMarking;
	}

	public static Set<Marking> apply(Set<Marking> markings, Map<Place, Place> map) {
		Set<Marking> appliedMarkings = new HashSet<Marking>();
		for (Marking marking : markings) {
			appliedMarkings.add(apply(marking, map));
		}
		return appliedMarkings;
	}
}
