/**
 * 
 */
package org.processmining.models.cnet;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
 * @author aadrians Oct 6, 2011
 * 
 */
public class CNetBinding {

	public static enum Type {
		/**
		 * represents an input binding
		 */
		INPUT,
		/**
		 * represents an output binding
		 */
		OUTPUT
	}

	/**
	 * Contains the bound activities of this binding
	 */
	private final Set<CNetNode> boundNodes;

	/**
	 * Points to the activity this binding belongs to
	 */
	private final CNetNode node;

	/**
	 * The type of the binding
	 */
	private final Type type;

	/**
	 * Instantiates a binding using CNetNodes.
	 * 
	 * @param net
	 * @param type
	 * @param node
	 * @param boundNodes
	 */
	public CNetBinding(Type type, CNetNode node, CNetNode... boundNodes) {
		this(type, node, Arrays.asList(boundNodes));
	}

	/**
	 * Instantiates a binding using CNetNodes.
	 * 
	 * @param net
	 * @param type
	 * @param node
	 * @param boundNodes
	 */
	public CNetBinding(Type type, CNetNode node, Collection<? extends CNetNode> boundNodes) {
		this.type = type;
		this.node = node;
		this.boundNodes = new HashSet<CNetNode>(boundNodes);
	}

	/**
	 * Returns the node to which this binding belongs
	 * 
	 * @return
	 */
	public CNetNode getNode() {
		return node;
	}

	/**
	 * returns the set of nodes bound by this binding
	 * 
	 * @return
	 */
	public Set<CNetNode> getBoundNodes() {
		return Collections.unmodifiableSet(boundNodes);
	}

	public boolean equals(Object o) {
		return (o instanceof CNetBinding) && ((CNetBinding) o).type == type && ((CNetBinding) o).node.equals(node)
				&& ((CNetBinding) o).boundNodes.equals(boundNodes);
	}

	public int hashCode() {
		return node.hashCode() + 37 * boundNodes.hashCode();
	}

	public Type getType() {
		return type;
	}

	public String toString() {
		return type.toString() + " of " + node.toString() + " : " + boundNodes.toString();
	}
}
