package org.processmining.plugins.properties.auxiliary;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;

/**
 * 
 * @author DSchunse
 *
 */
public class PerformanceOperations {
	public static Map<Double, Double> sum(Map<Double, Double> left, Map<Double, Double> right){
		HashMap<Double, Double> ret = new HashMap<Double, Double>();
		for(double i: left.keySet()){
			for(double j: right.keySet()){
				double value = 0d;
				if(ret.containsKey(i + j)){
					value = ret.get(i + j);
				}
				ret.put(i + j, value + (left.get(i) * right.get(j)));
			}
		}
		//return truncate(ret, 0.00005d);\
		return modifyGranularity(ret, 2);
		//return ret;
	}
	
	public static Map<Double, Double> union(Map<Double, Double> left, Double factorLeft, Map<Double, Double> right, Double factorRight){
		HashMap<Double, Double> ret = new HashMap<Double, Double>();
		for(double i: left.keySet()){
			ret.put(i, left.get(i) * factorLeft);
		}
		if(right != null){
			for(double i: right.keySet()){
				Double retValue = 0d;
				if(ret.containsKey(i)){
					retValue = ret.get(i);
				}
				ret.put(i, (right.get(i)*factorRight) + retValue);
			}
		}
		return ret;
	}
	
	public static Map<Double, Double> maximum(Map<Double, Double> left, Map<Double, Double> right){
		HashMap<Double, Double> ret = new HashMap<Double, Double>();
		for(double i: left.keySet()){
			for(double j: right.keySet()){
				Double value = 0d;
				if(ret.containsKey(Math.max(i, j))){
					value = ret.get(Math.max(i, j));
				}
				ret.put(Math.max(i, j), value + (left.get(i) * right.get(j)));
			}
		}
		return modifyGranularity(ret, 2);
	}
	
	public static <K extends Number> Double mean(Map<K, Double> set){
		double ret = 0d;
		for(K i: set.keySet()){
			ret += i.doubleValue() * set.get(i);
		}
		return ret;
	}
	
	public static Double variance(Map<Double, Double> set){
		double Cov = 0d;		
		for(double i: set.keySet()){
			Cov += (Math.pow(i, 2) * set.get(i));
		}
		return (Cov - Math.pow(mean(set), 2));
	}
	
	public static Double stdDev(Map<Double, Double> set){
		return Math.sqrt(variance(set));
	}
	
	public static boolean steadyStateRole(Map<Double, Double> set){
		for(double i: set.keySet()){
			if(i >= 1d || i < 0d){
				return false;
			}
		}
		return true;
	}
	
	public static boolean steadyStateAct(Map<Double, Double> set){
		for(double i: set.keySet()){
			if(i < 0d){
				return false;
			}
		}
		return true;
	}	
	
	@SuppressWarnings("unused")
	private static Map<Double, Double> truncate(Map<Double, Double> set, double threshold){
		HashMap<Double, Double> ret = new HashMap<Double, Double>();
		double removedProb = 0d;
		for(double i: set.keySet()){
			if(set.get(i) >= threshold){
				ret.put(i, set.get(i));
			}
			else{
				removedProb += set.get(i);
			}
		}
		return ret;
	}
	/**
	 * @param set the set to be sampled
	 * @param granularity the granularity of the returned set
	 * @param constant 1/constant is the probability to sample
	 * @return either set or the sampled set
	 */
	@SuppressWarnings("unused")
	private static Map<Double, Double> modifyGranularity(Map<Double, Double> set, int granularity, Integer constant){
		Random rand =  new Random();
		if(rand.nextInt() % constant == 0){
			return modifyGranularity(set, granularity);
		}
		else{
			return set;
		}
	}
	public static Map<Double, Double> modifyGranularity(Map<Double, Double> set, int granularity){
		// we have to estimate the variance and mean and create a new normal distribution
		return normalDistribution(mean(set), variance(set), granularity);
		//return set;
	}
	
	private static Map<Double, Double> scale(Map<Double, Double> set, double factor){
		for(double i: set.keySet()){
			set.put(i, set.get(i) * factor);
		}
		return set;
	}
	
	public static Double estimateVariance(Double mean, Double upper99){
		return Math.pow((upper99 - mean)/3, 2);
	}
	
	public static Map<Double, Double> normalDistributionUpper99(Double mean, Double upper99, int granularity){
		return normalDistribution(mean, estimateVariance(mean, upper99), granularity);
	}
	
	public static Map<Double, Double> normalDistribution(double mean, double variance, int granularity){
		HashMap<Double, Double> ret = new HashMap<Double, Double>();

		if(variance == 0d){
			ret.put(mean, 1d);
			return ret;
		}
		else{
			// what do we want?
			// - mean
			// - granularty amount of values between mean and 3\sigma
			double sigma = Math.sqrt(variance);
			double stepSize = (3*sigma) / granularity;
			double sumProbability = 0d;
			// TODO there is something wrong with the bounds due to rounding errors -> wrong variance, skewed towards lower values
			for(int i = -granularity; i <= granularity; i++){
				double prob = giveProbability(mean, variance, mean + (i * stepSize));
				sumProbability += prob;
				ret.put(mean + (i * stepSize), prob);
			}
			// we still need to normalise
			return scale(ret, 1 / sumProbability);
		}
	}
	
	public static double giveProbability(Double mean, Double variance, Double value){
		// lets encode the formula for the normal distribution
		// f(x) = see http://en.wikipedia.org/wiki/Normal_distribution
		return (Math.pow(Math.E, -(Math.pow(value - mean, 2)/ (2*variance))) / (Math.sqrt(variance) * Math.sqrt(2 * Math.PI)));
	}
}
