package org.processmining.plugins.properties.processmodel.configurable.abstractproperty;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.processmodel.configurable.ConfigurableProperty;
import org.processmining.plugins.properties.processmodel.configurable.ConfigurablePropertyConfiguration;

/**
 * 
 * @author DSchunse
 *
 * @param <K>
 */
@SuppressWarnings("serial")
public abstract class ConfigurablePropertySingleElement<K> extends ConfigurablePropertyList<K>{
	public Pair<List<K>, List<K>> getDefaultValue() {
		return new Pair<List<K>, List<K>>(new ArrayList<K>(), new ArrayList<K>());
	}
	
	@SuppressWarnings("unchecked")
	public boolean isValidConfiguration(Object elements, ConfigurablePropertyConfiguration<List<K>> config, Map<Class<? extends ConfigurableProperty<?>>, ?> dependentProperties){
		Pair<List<K>, List<K>> elems = (Pair<List<K>, List<K>>) elements;
		return super.isValidConfiguration(elements, config, dependentProperties) && config.getNonRemoved().size() <= 1 && config.getRemoved().size() < elems.getFirst().size();
	}
	
	@SuppressWarnings("unchecked")
	public Object instantiateTotal(Object elements, ConfigurablePropertyConfiguration<List<K>> config) {
		// we need to check if this is a total configuration, if so, then return a single value to be used
		Pair<List<K>, List<K>> elems = (Pair<List<K>, List<K>>) elements;
		if(this.isTotalConfiguration(elements, config)){
			return ((List<K>)super.instantiateTotal(elems, config)).get(0);
		}
		else{
			throw new NullPointerException("Not a complete configuration");
		}
	}
	
	public Pair<List<K>, List<K>> clone(Object element, Class<? extends ConfigurablePropertyList<K>> superClass) {
		try{
			return superClass.newInstance().clone(element);
		} catch (Exception e){
			e.printStackTrace();
		}
		return null;
	}

	public Object unmarshall(String xml, Class<? extends ConfigurablePropertyList<K>> superClass) {
		try{
			return superClass.newInstance().unmarshall(xml);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}	
	

	public Object makeNonConfigurable(Object elements) {
		@SuppressWarnings("unchecked")
		Pair<List<K>, List<K>> values = (Pair<List<K>, List<K>>) elements;
		return values.getFirst().get(0);
	}

	@SuppressWarnings("unchecked")
	public Object makeConfigurable(Object element) {
		Pair<List<K>, List<K>> values = new Pair<List<K>, List<K>>(new ArrayList<K>(), new ArrayList<K>());
		values.getFirst().add((K) element);
		return values;
	}
}
