package org.processmining.plugins.properties.processmodel.gui;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;

import javax.swing.JComponent;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.plugins.properties.processmodel.PropertableElement;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.plugins.properties.processmodel.gui.abstractguiproperty.AddButton;

/**
 * 
 * @author DSchunse
 *
 * @param <K>
 */
public interface GUIProperty<K> {
	/**
	 * Sets a particular property, note we need to have the actual property to be able to obtain the header, and data rows
	 * @param context
	 * @param tree
	 * @param property
	 * @return
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 */
	@Deprecated
	public <E extends PropertableElement, P extends Property<?>> void initialiseGUIProperty(UIPluginContext context, E model, Collection<E> elements, P property, HashMap<E, Object> mapping, boolean independent) throws InstantiationException, IllegalAccessException;
	
	/**
	 * Generic way to get the component for a collection of elements and a property, the editable is for view/edit functionality 
	 * @param elements
	 * @param property
	 * @param elementLines
	 * @param editable
	 * @return
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	public <E extends PropertableElement, P extends Property<?>> JComponent initialiseTabPane(Collection<E> elements, P property, Object elementLines, boolean editable, boolean independent) throws InstantiationException, IllegalAccessException;
	
	/**
	 * Generic way to get the component for an element and a property, the editable is for view/edit functionality
	 * @param element
	 * @param property
	 * @param elementLines
	 * @param editable
	 * @param independent
	 * @return
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	public <E extends PropertableElement, P extends Property<?>> JComponent initialiseTabPane(E element, P property, Object elementLines, boolean editable, boolean independent) throws InstantiationException, IllegalAccessException;
	/**
	 * Fill the GUI with the values of the property
	 * @param context
	 * @param prop
	 * @param element
	 * @param button
	 * @param dataRows
	 * @param independent
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	public <E extends PropertableElement, P extends Property<?>> void fillGUI(P prop, E element, AddButton button, List<List<JComponent>> dataRows, boolean independent) throws InstantiationException, IllegalAccessException;
	
	/**
	 * The header row(s) for a property
	 * @param context
	 * @param tree
	 * @return
	 */
	public <E extends PropertableElement> List<List<JComponent>> getHeaderRows(E element);
	
	/**
	 * The data row for a property, i.e., which things will be added when the "add" button is pressed	
	 * @param context
	 * @param tree
	 * @return
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 */
	public <E extends PropertableElement> List<List<JComponent>> getDataRow(E element) throws InstantiationException, IllegalAccessException;
	
	/**
	 * The footer row(s) for a property	
	 * @param context
	 * @param tree
	 * @return
	 */
	public <E extends PropertableElement> List<List<JComponent>> getFooterRows(E element);
	
	
	/**
	 * Parse the components and make appropriate data structures out of it
	 * @param lines
	 * @return
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 */
	public <P extends Property<?>> K parseComponents(P property, List<List<List<Object>>> lines) throws InstantiationException, IllegalAccessException;
}
