package org.processmining.plugins.properties.processmodel.impl;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import javax.swing.ImageIcon;

import org.processmining.framework.plugin.annotations.KeepInProMCache;
import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.annotations.OriginatorPerspective;
import org.processmining.plugins.properties.annotations.PropertableElementProperty;
import org.processmining.plugins.properties.enumerations.Constructs;
import org.processmining.plugins.properties.processmodel.abstractproperty.HighLevelProperty;
import org.processmining.plugins.properties.processmodel.gui.GUIProperty;
import org.processmining.plugins.properties.processmodel.gui.impl.GUIPropertyYerkesDodsonInterval;

/**
 * 
 * @author DSchunse
 *
 */

@KeepInProMCache
@OriginatorPerspective
@PropertableElementProperty
public class YerkesDodsonInterval extends HighLevelProperty<HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>>> {

	private static final long serialVersionUID = -8348569829190729351L;

	public Long getID() {
		return serialVersionUID;
	}

	public String getName() {
		return "YerkesDodsonInterval";
	}
	
	@SuppressWarnings("unchecked")
	public HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>> clone(Object element){
		HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>> elem = (HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>>) element;
		HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>> ret = new HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>>();
		for(UUID id: elem.keySet()){
			HashMap<Pair<Double, Double>, Distribution> elems = new HashMap<Pair<Double, Double>, Distribution>();
			for(Pair<Double, Double> dd: elems.keySet()){
				elems.put(new Pair<Double, Double>(new Double(dd.getFirst()), dd.getSecond()), new Distribution(elem.get(id).get(dd)));
			}
			ret.put(UUID.fromString(id.toString()), elems);
		}
		return ret;
	}

	public int compare(Object o1, Object o2) {
		return 0;
	}

	public HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>> getDefaultValue() {
		return null;
	}

	public double getValueDouble(Object value) {
		return 0;
	}

	public ImageIcon getIcon() {
		return null;
	}

	public boolean hasOrdering() {
		return false;
	}

	public boolean higherBetter() {
		return false;
	}

	@SuppressWarnings("unchecked")
	public String marshall(Object value) {
		HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>> yerkesD = (HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>>) value;
		// so lets peal off the different constructs
		String ret = ""; //"<valueEntry>\n";
		for(UUID id: yerkesD.keySet()){
			// we have to write the originator
			//ret += "<valueEntryOrg id=\"" + id + "\">\n";
			// now add the different things
			for(Pair<Double, Double> key: yerkesD.get(id).keySet()){
				ret += "<valueEntry id=\"" + id + "\" first=\"" + key.getFirst() + "\" second=\"" + key.getSecond() + "\" " + yerkesD.get(id).get(key).distributionValue() + "/>\n";
			}
			//ret += "</valueEntryOrg>\n";
		}
		//ret += "</valueEntry>\n";
		return ret;
	}

	public Object unmarshall(String xml) {
		// lets have some fun with the XML schema :-)
		HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>> ret = new HashMap<UUID, HashMap<Pair<Double,Double>,Distribution>>();
		String[] entries = xml.split("<valueEntry");
		// so we have the information per originator
		for(String entry: entries){
			if(!entry.isEmpty()){
				// lets get the id from this: does anyone have an id...? :-P
				UUID id = UUID.fromString(entry.split("id=\"")[1].split("\"")[0]);
				// so we have the id, lets split the others
				// so the values are split
				Double first = Double.valueOf(entry.split("first=")[1].split("\"")[1]);
				Double second = Double.valueOf(entry.split("second=")[1].split("\"")[1]);
				Distribution distr = new Distribution();
				HashMap<Pair<Double, Double>, Distribution> interval = ret.get(id);
				if(interval == null){
					interval = new HashMap<Pair<Double,Double>, Distribution>();
				}
				interval.put(new Pair<Double, Double>(first, second), new Distribution(entry.split("second=\"\\d+.\\d+\"")[1]));
				ret.put(id, interval);
			}
		}
		return ret;
	}

	public GUIProperty<HashMap<UUID, HashMap<Pair<Double, Double>, Distribution>>> getGUIProperty() throws InstantiationException, IllegalAccessException {
		return GUIPropertyYerkesDodsonInterval.class.newInstance();
	}

	public Set<Constructs> getMeaningfulTypeConstructs() {
		Set<Constructs> constructs = new HashSet<Constructs>();
		constructs.add(Constructs.ManualTask);
		return constructs;
	}	
	
}
