package org.processmining.pnanalysis.metrics;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.processmining.pnanalysis.metrics.impl.PetriNetCardosoMetric;
import org.processmining.pnanalysis.metrics.impl.PetriNetCyclomaticMetric;
import org.processmining.pnanalysis.metrics.impl.PetriNetDensityMetric;
import org.processmining.pnanalysis.metrics.impl.PetriNetNofArcsMetric;
import org.processmining.pnanalysis.metrics.impl.PetriNetNofPlacesMetric;
import org.processmining.pnanalysis.metrics.impl.PetriNetNofTransitionsMetric;
import org.processmining.pnanalysis.metrics.impl.PetriNetStructurednessMetric;


public class PetriNetMetricManager {

	private static PetriNetMetricManager instance = null;
	private List<PetriNetMetric> metrics;
	private PetriNetMetric defaultMetric;
	private boolean sorted;

	private PetriNetMetricManager() {
		metrics = new ArrayList<PetriNetMetric>();
		defaultMetric = null;
		sorted = false;
		register(new PetriNetCardosoMetric(), false);
		register(new PetriNetCyclomaticMetric(), false);
		register(new PetriNetDensityMetric(), true);
		register(new PetriNetNofArcsMetric(), false);
		register(new PetriNetNofPlacesMetric(), false);
		register(new PetriNetNofTransitionsMetric(), false);
		register(new PetriNetStructurednessMetric(), false);
	}

	public static PetriNetMetricManager getInstance() {
		if (instance == null) {
			instance = new PetriNetMetricManager();
		}
		return instance;
	}
	
	public void register(PetriNetMetric metric, boolean isDefault) {
		metrics.add(metric);
		if (isDefault) {
			defaultMetric = metric;
		}
		sorted = false;
	}

	public List<PetriNetMetric> getMetrics() {
		if (!sorted) {
			Collections.sort(metrics, new Comparator<PetriNetMetric>() {
				public int compare(PetriNetMetric m1, PetriNetMetric m2) {
					return m1.getName().compareTo(m2.getName());
				}
			});
			sorted = true;
		}
		return metrics;
	}
	
	public boolean isDefault(PetriNetMetric miner) {
		return miner == defaultMetric;
	}
	
	public PetriNetMetric getMetric(String name) {
		if (name != null) {
			for (PetriNetMetric metric : metrics) {
				if (name.equals(metric.getName())) {
					return metric;
				}
			}
		}
		return defaultMetric;
	}
}
