package org.processmining.pnanalysis.semantics;

import java.util.Collection;
import java.util.HashSet;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.ExecutionInformation;
import org.processmining.models.semantics.IllegalTransitionException;
import org.processmining.models.semantics.Semantics;
import org.processmining.models.semantics.petrinet.CTMarking;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.models.semantics.petrinet.PetrinetSemantics;
import org.processmining.models.semantics.petrinet.impl.PetrinetSemanticsFactory;

public class CyclomaticSemantics implements PetrinetSemantics {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1909845346796187378L;
	private Semantics<Marking, Transition> semantics;
	private boolean isUnbounded;
	private int nofStates;

	public CyclomaticSemantics(int nofStates) {
		this.nofStates = nofStates;
	}
	
	public void setCurrentState(Marking currentState) {
		semantics.setCurrentState(currentState);
	}

	public Marking getCurrentState() {
		return semantics.getCurrentState();
	}

	public Collection<Transition> getExecutableTransitions() {
		if (isUnbounded || nofStates < 0) {
			/*
			 * Do not expand, is unbounded already.
			 */
			return new HashSet<Transition>();
		}
		Marking marking = semantics.getCurrentState();
		if (marking instanceof CTMarking) {
			CTMarking ctMarking = (CTMarking) marking;
			if (ctMarking.hasOmegaPlace()) {
				isUnbounded = true;
				/*
				 * No transition enabled if omega marking.
				 */
				return new HashSet<Transition>();
			}
		}
		return semantics.getExecutableTransitions();
	}

	public ExecutionInformation executeExecutableTransition(Transition toExecute) throws IllegalTransitionException {
		nofStates--;
		return semantics.executeExecutableTransition(toExecute);
	}

	public void initialize(Collection<Transition> transitions, Marking initialState) {
		semantics = PetrinetSemanticsFactory.regularPetrinetSemantics(Petrinet.class);
		semantics.initialize(transitions, initialState);
		isUnbounded = false;
	}

}

