package org.processmining.processtree.configuration;

import java.util.BitSet;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.Stack;
import java.util.UUID;

import org.processmining.processtree.Block;
import org.processmining.processtree.Block.And;
import org.processmining.processtree.Block.DefLoop;
import org.processmining.processtree.Block.Seq;
import org.processmining.processtree.Block.XorLoop;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;

public class ConfigurationAuxiliary {
	public static void nextBitSequece(BitSet currentConfiguration){
		// we have to add one to the bitsequence
		boolean carry = true;
		for(int i = 0; i < currentConfiguration.length() - 1; i++){
			boolean tempcarry = currentConfiguration.get(i) && carry;
			currentConfiguration.set(i, (currentConfiguration.get(i) || carry) && !tempcarry);
			carry = tempcarry;
		}
	}
	
	public static <E extends Object> boolean overlap(Set<E> set1, Set<E> set2){
		for(E element: set2){
			if(set1.contains(element)){
				return true;
			}
		}
		return false;
	}
	
	public static <E extends Object> boolean overlap(E element, Set<E> set2){
		return set2.contains(element);
	}
	
	public static boolean smaller(BitSet left, BitSet right) throws Exception{
		if(right == null){
			// special case, this way we do not need to explicitly model the maximal bitsequence possible
			return true;
		}
		if(left.size() != right.size()){
			// bit weird... let's throw an exception
			throw new Exception("Bit Sequences cannot be different in size\n");
		}
		else{
			for(int i = left.size() - 1; i >= 0; i--){
				if(left.get(i) && !right.get(i)){
					return false;
				}
				if(!left.get(i) && right.get(i)){
					return true;
				}
			}
			// they are the same, so it is not smaller
			return false;
		}
	}
	
	// TODO: test me
	public static Map<UUID, Integer> computeClosureBlockingMultiSet(ProcessTree tree, Set<UUID> blockedEdges){
		Map<UUID, Integer> blockedEdgesMultiSet = new HashMap<UUID, Integer>();
		Stack<UUID> unprocessedEdges = new Stack<UUID>();
		unprocessedEdges.addAll(blockedEdges);
		while(!unprocessedEdges.isEmpty()){
			UUID e = unprocessedEdges.pop();
			blockedEdgesMultiSet.put(e, blockedEdgesMultiSet.get(e) == null ? 1 : blockedEdgesMultiSet.get(e) + 1);
			Edge treeEdge = tree.getEdge(e);
			if(treeEdge == null){
				//we are done, this edge no longer exists
				break;
			}
			Block blockSource = (Block) tree.getNode(treeEdge.getSource().getID());
			if(blockSource == null){
				break;
			}
			// check if all of its children are blocked
			boolean allChildrenBlocked = true;
			for(Edge eChild: blockSource.getOutgoingEdges()){
				if(!blockedEdgesMultiSet.containsKey(eChild) && !e.equals(eChild.getID())){
					allChildrenBlocked = false;
				}
			}
			if(allChildrenBlocked){
				for(Edge incom: blockSource.getIncomingEdges()){
					unprocessedEdges.push(incom.getID());
				}
			}
				
			// check if the parent is one of SEQ, AND, LOOP
			if(blockSource instanceof And || blockSource instanceof Seq){
				// now block the incoming edges of this node
				for(Edge incom: blockSource.getIncomingEdges()){
					unprocessedEdges.push(incom.getID());
				}
			}
			if(blockSource instanceof DefLoop || blockSource instanceof XorLoop){
				// only move blocked up, if the 1st, or 3rd is blocked
				if(blockSource.getOutgoingEdges().indexOf(treeEdge) != 1){
					for(Edge incom: blockSource.getIncomingEdges()){
						unprocessedEdges.push(incom.getID());
					}
				}
			}
		}
		// we now have the upward closure, now make it downward
		int sizeOfBlocked = blockedEdges.size();
		int sizeOfBlockedOld = 0;
		while(sizeOfBlockedOld != sizeOfBlocked){
			sizeOfBlockedOld = sizeOfBlocked;
			for(Node n: tree.getNodes()){
				if(tree.getRoot().equals(n)){
					// we have the root, forall over an empty domain, so skip it
				}
				else{
					boolean allBlocked = true;
					for(Edge e: n.getIncomingEdges()){
						if(!blockedEdgesMultiSet.containsKey(e)){
							allBlocked = false;
							break;
						}
					}
					if(allBlocked && n instanceof Block){
						// add all the outgoing edges to this
						for(Edge incom: ((Block)n).getOutgoingEdges()){
							blockedEdgesMultiSet.put(incom.getID(), blockedEdgesMultiSet.get(incom.getID()) == null ? 1 : blockedEdgesMultiSet.get(incom.getID()) + 1);
							sizeOfBlocked++;
						}
					}
				}
			}
		}
		return blockedEdgesMultiSet;
	}
	
	public static Set<UUID> computeClosureBlocking(ProcessTree tree, Set<UUID> blockedEdges){
		//return new HashSet<UUID>(computeClosureBlockingMultiSet(tree, blockedEdges).keySet());
		int sizeOfBlocked = blockedEdges.size();
		int sizeOfBlockedOld = 0;
		Set<UUID> blockedEdgesRet = new HashSet<UUID>(blockedEdges);
		while(sizeOfBlockedOld != sizeOfBlocked){
			sizeOfBlockedOld = sizeOfBlocked;
			Set<UUID> blockedEdgesToAdd = new HashSet<UUID>();
			for(UUID e: blockedEdgesRet){
				Edge treeEdge = tree.getEdge(e);
				if(treeEdge == null){
					//we are done, this edge no longer exists
					break;
				}
				Block blockSource = (Block) tree.getNode(treeEdge.getSource().getID());
				if(blockSource == null){
					break;
				}
				// check if all of its children are blocked
				boolean allChildrenBlocked = true;
				for(Edge eChild: blockSource.getOutgoingEdges()){
					if(!blockedEdgesRet.contains(eChild.getID()) && !blockedEdgesToAdd.contains(eChild.getID())){
						allChildrenBlocked = false;
					}
				}
				if(allChildrenBlocked){
					for(Edge incom: blockSource.getIncomingEdges()){
						blockedEdgesToAdd.add(incom.getID());
					}
				}
				
				// check if the parent is one of SEQ, AND, LOOP
				// degenerate case when the seq, and, loop is the root, in that case we do not have incoming edges
				if(blockSource instanceof And || blockSource instanceof Seq){
					// now block the incoming edges of this node
					for(Edge incom: blockSource.getIncomingEdges()){
						blockedEdgesToAdd.add(incom.getID());
					}
					if(tree.getRoot().equals(blockSource)){
						for(Edge outGoing: blockSource.getOutgoingEdges()){
							blockedEdgesToAdd.add(outGoing.getID());
						}
					}
				}
				if(blockSource instanceof DefLoop || blockSource instanceof XorLoop){
					// only move blocked up, if the 1st, or 3rd is blocked
					if(blockSource.getOutgoingEdges().indexOf(treeEdge) != 1){
						for(Edge incom: blockSource.getIncomingEdges()){
							blockedEdgesToAdd.add(incom.getID());
						}
						if(tree.getRoot().equals(blockSource)){
							for(Edge outGoing: blockSource.getOutgoingEdges()){
								blockedEdgesToAdd.add(outGoing.getID());
							}
						}
					}
				}
			}
			blockedEdgesRet.addAll(blockedEdgesToAdd);
			sizeOfBlocked = blockedEdgesRet.size();
		}
		// we now have the upward closure, now make it downward
		sizeOfBlocked = blockedEdges.size();
		sizeOfBlockedOld = 0;
		while(sizeOfBlockedOld != sizeOfBlocked){
			sizeOfBlockedOld = sizeOfBlocked;
			for(Node n: tree.getNodes()){
				if(tree.getRoot().equals(n)){
					// we have the root, forall over an empty domain, so skip it
				}
				else{
					boolean allBlocked = true;
					for(Edge e: n.getIncomingEdges()){
						if(!blockedEdgesRet.contains(e)){
							allBlocked = false;
							break;
						}
					}
					if(allBlocked && n instanceof Block){
						// add all the outgoing edges to this
						for(Edge incom: ((Block)n).getOutgoingEdges()){
							blockedEdgesRet.add(incom.getID());
						}
					}
				}
			}
			sizeOfBlocked = blockedEdgesRet.size();
		}
		return blockedEdgesRet;
	}
}
