package org.processmining.processtree.configuration.controlflow.impl;

import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.processmining.processtree.configuration.controlflow.ControlFlowConfiguration;

public class ControlFlowConfigurationImport {
	
	public static String blocked = ControlFlowConfigurationExport.blocked;
	public static String notBlocked = ControlFlowConfigurationExport.notBlocked;
	public static String hidden = ControlFlowConfigurationExport.hidden;
	public static String notHidden = ControlFlowConfigurationExport.notHidden;
	public static String replacement = ControlFlowConfigurationExport.replacement;
	public static String notReplacement = ControlFlowConfigurationExport.notReplacement;
	
	public static ControlFlowConfiguration importer(String xml){
		ControlFlowConfiguration cfConf = new ControlFlowConfigurationImpl();
		// let's remove the opening and closing tag
		String xmlSplit = xml.split("<ControlFlowConfiguration>")[1].split("</ControlFlowConfiguration>")[0];
		// let's split accordingly
		cfConf.addBlockedEdges(split(blocked, xmlSplit));
		cfConf.addNonBlockedEdges(split(notBlocked, xmlSplit));
		cfConf.addHiddenEdges(split(hidden, xmlSplit));
		cfConf.addNonHiddenEdges(split(notHidden, xmlSplit));
		importSelectedElements(cfConf, xmlSplit);
		importNonSelectedElements(cfConf, xmlSplit);
		return cfConf;
	}
	
	private static Set<UUID> split(String keyword, String xml){
		Set<UUID> ret = new HashSet<>();
		// let's split on the keyword
		String values = xml.split("<" + keyword + ">")[1].split("</" + keyword + ">")[0];
		// now split on the IDS
		String[] ids = values.split("<id=\"");
		for(String id: ids){
			id = id.trim();
			if(!id.isEmpty()){
				ret.add(UUID.fromString(id.split("\"/>")[0]));
			}
		}
		return ret;
	}
	
	private static void importSelectedElements(ControlFlowConfiguration cfConfiguration, String xml){
		String importString = xml.split("<" + replacement + ">")[1].split("</" + replacement + ">")[0];
		String idVals[] = importString.split("<id=\"");
		for(String idVal: idVals){
			if(!idVal.isEmpty()){
				String key = idVal.split("\" value")[0];
				String value = idVal.split(" value=\"")[1].split("\"/>")[0];
				cfConfiguration.setSelectedReplacement(UUID.fromString(key), UUID.fromString(value));
			}
		}
	}
	
	private static void importNonSelectedElements(ControlFlowConfiguration cfConfiguration, String xml){
		String importString = xml.split("<" + notReplacement + ">")[1].split("</" + notReplacement + ">")[0];
		String idVals[] = importString.split("<id=\"");
		for(String idVal: idVals){
			if(!idVal.isEmpty()){
				String key = idVal.split("\" value")[0];
				String value = idVal.split(" value=\"")[1].split("\"/>")[0];
				cfConfiguration.addNonSelectedReplacement(UUID.fromString(key), UUID.fromString(value));
			}
		}
	}
}
