package org.processmining.processtree.conversion.protos;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.DecimalFormat;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.OsUtil;
import org.processmining.plugins.properties.conversion.YerkesDodsonInterval2TaskDistribution;
import org.processmining.plugins.properties.enumerations.DistributionType;
import org.processmining.plugins.properties.enumerations.OROptions;
import org.processmining.plugins.properties.processmodel.impl.ArrivalProcess;
import org.processmining.plugins.properties.processmodel.impl.Costs;
import org.processmining.plugins.properties.processmodel.impl.Distribution;
import org.processmining.plugins.properties.processmodel.impl.NrOfResources;
import org.processmining.plugins.properties.processmodel.impl.ORSemantic;
import org.processmining.plugins.properties.processmodel.impl.OutgoingProbability;
import org.processmining.plugins.properties.processmodel.impl.SimulationProperty;
import org.processmining.plugins.properties.processmodel.impl.TaskDistribution;
import org.processmining.plugins.properties.processmodel.impl.YerkesDodsonInterval;
import org.processmining.processtree.Block;
import org.processmining.processtree.Block.And;
import org.processmining.processtree.Block.Def;
import org.processmining.processtree.Block.DefLoop;
import org.processmining.processtree.Block.Or;
import org.processmining.processtree.Block.Seq;
import org.processmining.processtree.Block.Xor;
import org.processmining.processtree.Block.XorLoop;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Event;
import org.processmining.processtree.Node;
import org.processmining.processtree.Originator;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.ProcessTreeElement;
import org.processmining.processtree.Task;
import org.processmining.processtree.Task.Manual;

/*
 * TODO we have to add argument support for the translation of the OR
 * 
 * SPECIAL CASES: only an activity, sharing of subtrees is not desirable
 * SPECIAL CASES: we cannot split on a subprocess
 * 
 * ISSUES: Constant with a different value does not work
 * ISSUES: BPM|One cannot handle, but allows you to specify, floats for all things.... pffff....
 * ISSUES: Events do not work, so we skip them in the transformation
 * ISSUES: we multiply probabilities with a constant just to make it work
 * 
 */
@Plugin(name = "ProcessTree2Protos export", returnLabels = {}, returnTypes = {}, parameterLabels = { "ProcessTree", "File" }, userAccessible = true)
@UIExportPlugin(description = "ProcessTree2Protos Export", extension = "xml")
@Deprecated
public class ProcessTree2Protos {
	
	@PluginVariant(variantLabel = "Protos Export", requiredParameterLabels = { 0, 1 })
	@Deprecated
	public void exportModel(UIPluginContext context, ProcessTree tree, File f) throws Exception{
		FileWriter file = new FileWriter(f);
		convertToProtos(context, tree, file);
		
		File directory = OsUtil.getProMPackageDirectory();
		// now we have the xml, but we still need to transform this file to a proper .pal file
		// check if the user has specific transformation code
		File protosLocation = new File(directory + "/protos/");
		if(protosLocation.exists()){
			FileWriter palFile = new FileWriter(f.getParentFile() + "//" + f.getName().substring(0, f.getName().length() - 4) + ".pal");
			Process p = Runtime.getRuntime().exec(new String[]{"javaw", "-Xmx1000m", "-jar", protosLocation.getAbsolutePath() + "/saxon9.jar", f.getAbsolutePath(), protosLocation.getAbsolutePath() + "/xmlimport.xslt"});
			InputStream stdin = p.getInputStream();
			BufferedReader br = new BufferedReader(new InputStreamReader(stdin));
			String line;
			while((line = br.readLine()) != null){
				palFile.write(line + "\n");
			}
			palFile.close();
			p.waitFor();
		}
	}
	
	public void convertToProtos(PluginContext context, ProcessTree tree, FileWriter file) throws Exception{
		System.out.print("Exporting to Protos...");
		file.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
		file.write("<protosmodel xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"protosmodel.xsd\">\n");
		file.write("<name>" + tree.getName() + "</name>\n");
		file.write("<rolegraph>\n");
		for(Originator o: tree.getOriginators()){
			writeRole(tree, o, file);
		}
		// TODO: hierarchy stub
		file.write("</rolegraph>\n");
		// TODO: group stub
		
		// now convert the process model
		writeNode(tree, tree.getRoot(), file);
		
		writeOptions(tree, file);
		
		file.write("</protosmodel>\n");
		file.close();
		System.out.print("DONE\n");
	}
	
	private void writeRole(ProcessTree tree, Originator o, FileWriter file) throws IOException, InstantiationException, IllegalAccessException{
		//if(o instanceof Role){
			file.write("<role id=\"" + "r" + getID(o) + "\" organisation=\"internal\">\n");
			file.write("<name>" + o.getName() + "_" + getID(o) + "</name>\n");
			if(o.getIndependentProperty(NrOfResources.class) != null){
				file.write("<available>" + o.getIndependentProperty(NrOfResources.class) + "</available>\n");
			}
			else{
				file.write("<available>" + NrOfResources.class.newInstance().getDefaultValue() + "</available>\n");
			}
			if(o.getIndependentProperty(Costs.class) != null){
				file.write("<cost>" + toStringObject(o.getIndependentProperty(Costs.class)) + "</cost>\n");
			}
			else{
				file.write("<cost>" + toString(Costs.class.newInstance().getDefaultValue()) + "</cost>\n");
			}
			file.write("<description></description>\n");
			file.write("<workinstruction></workinstruction>\n");
			file.write("<deactivate>false</deactivate>\n");
			file.write("</role>\n");
		//}
	}
	
	private void writeNode(ProcessTree tree, Node n, FileWriter file) throws IOException, InstantiationException, IllegalAccessException{
		// we have a node and now we need to write it
		if(n instanceof Task){
			// special case if the activity is the root node
			if(tree.getRoot().equals(n)){
				file.write("<subprocess id=\"p" + getID(n) + "\">\n");
				file.write("<name>" + n.getName() + getID(n) + "</name>\n");
				file.write("<export>true</export>\n");
				file.write("<manuallayout>false</manuallayout>\n");
				file.write("<description></description>\n");
				file.write("<workinstruction></workinstruction>\n");
				file.write("<processflow>\n");
				String startID = "s_" + getID(n);
				String endID = "e_" + getID(n);
				String startStatusID = "s_" + getID(n) + "status";
				String endStatusID = "e_" + getID(n) + "status";
				
				writeAct(tree, null, startID, file, false, true, true, false, false, "AND", "AND");
				writeAct(tree, n, "", file, false, false, false, false, false, "AND", "AND");
				writeAct(tree, null, endID, file, false, false, false, true, true, "AND", "AND");
				writeStatus(startStatusID, file);
				writeStatus(endStatusID, file);
				writeArc(startID, startStatusID, 1d, file);
				writeArc(startStatusID, "a" + getID(n), 1d, file);
				writeArc("a" + getID(n), endStatusID, 1d, file);
				writeArc(endStatusID, endID, 1d, file);
				file.write("</processflow>\n");
				file.write("<data></data>\n");
				file.write("</subprocess>\n");
			}
		}
		else if(n instanceof Event){
			// no events are supported in Protos so SKIP
			if(tree.getRoot().equals(n)){
				Node child = ((Event) n).getChildren().get(0);
				Node c = ((Event) n).getChildren().get(0);
				while(c instanceof Event){
					child = ((Event)c).getChildren().get(0);
					c = child;
				}
				if(child instanceof Task){
					// write the activity as specified earlier
					file.write("<subprocess id=\"p" + getID(child) + "\">\n");
					file.write("<name>" + child.getName() + getID(child) + "</name>\n");
					file.write("<export>true</export>\n");
					file.write("<manuallayout>false</manuallayout>\n");
					file.write("<description></description>\n");
					file.write("<workinstruction></workinstruction>\n");
					file.write("<processflow>\n");
					String startID = "s_" + getID(child);
					String endID = "e_" + getID(child);
					String startStatusID = "s_" + getID(child) + "status";
					String endStatusID = "e_" + getID(child) + "status";
					
					writeAct(tree, null, startID, file, false, true, true, false, false, "AND", "AND");
					writeAct(tree, child, "", file, false, false, false, false, false, "AND", "AND");
					writeAct(tree, null, endID, file, false, false, false, true, true, "AND", "AND");
					writeStatus(startStatusID, file);
					writeStatus(endStatusID, file);
					writeArc(startID, startStatusID, 1d, file);
					writeArc(startStatusID, "a" + getID(child), 1d, file);
					writeArc("a" + getID(child), endStatusID, 1d, file);
					writeArc(endStatusID, endID, 1d, file);
					file.write("</processflow>\n");
					file.write("<data></data>\n");
					file.write("</subprocess>\n");
				}
				else{
					writeNode(tree, child, file);
				}
			}
			else{
				writeNode(tree, ((Event) n).getChildren().get(0), file);
			}

		}
		else{
			// now split to operator types
			file.write("<subprocess id=\"p" + getID(n) + "\">\n");
			file.write("<name>" + n.getName() + getID(n) + "</name>\n");
			file.write("<export>true</export>\n");
			file.write("<manuallayout>false</manuallayout>\n");
			file.write("<description></description>\n");
			file.write("<workinstruction></workinstruction>\n");
			file.write("<processflow>\n");
			// make a start and end activity, and based on type we need to do different constructs
			String startID = "s_" + getID(n);
			String endID = "e_" + getID(n);
			String midID = "m_" + getID(n);
			String specialID = "sp_" + getID(n);
			if(n instanceof Seq ||
					n instanceof And ||
					n instanceof DefLoop || 
					n instanceof XorLoop
					// TODO: also change due to Exspect
					//||
					//n instanceof Def
					){
				writeAct(tree, null, startID, file, false, true, tree.getRoot().equals(n), false, false, "AND", "AND");
				writeAct(tree, null, endID, file, false, false, false, true, tree.getRoot().equals(n), "AND", "AND");
				// TODO: also changes due to Exspect
				if(n instanceof XorLoop
						// TODO: start of change
					|| n instanceof DefLoop
						){
					writeAct(tree, null, midID, file, false, false, false, false, false, "XOR", "AND");
				}
			}
			// TODO: also changes due to Exspect
			else if(n instanceof Xor
					// TODO: start of change
					|| n instanceof Def
					){
				writeAct(tree, null, startID, file, false, true, tree.getRoot().equals(n), false, false, "XOR", "AND");
				writeAct(tree, null, endID, file, false, false, false, true, tree.getRoot().equals(n), "AND", "XOR");	
			}
			else{
				// we have an OR do something smart
				// so the plan is: 
				// - we have a control place for first time execution
				// - a control place for second time execution
				// - 3 silent transitions per subprocess (CHECK)
				writeAct(tree, null, startID, file, false, true, tree.getRoot().equals(n), false, false, "AND", "AND");
				writeAct(tree, null, endID, file, false, false, false, true, tree.getRoot().equals(n), "AND", "AND");
				writeAct(tree, null, specialID, file, false, false, false, false, false, "AND", "AND");
				for(Node c: ((Block)n).getChildren()){
					writeAct(tree, null, "tau1" + getID(c), file, false, false, false, false, false, "XOR", "AND");
					writeAct(tree, null, "tau2" + getID(c), file, false, false, false, false, false, "AND", "AND");
					writeAct(tree, null, "tau3" + getID(c), file, false, false, false, false, false, "AND", "AND");
					writeAct(tree, null, "tau4" + getID(c), file, false, false, false, false, false, "AND", "AND");
				}
			}
			// write the individual children, note that we need to write different things when we are writing an activity and a subprocess
			for(Node c: ((Block)n).getChildren()){
				Node child = c;
				if(c instanceof Event){
					// take the child
					// can be multiple event nodes, take transitive closure
					while(c instanceof Event){
						child = ((Event) c).getChildren().get(0);
						c = child;
					}
				}
				if(child instanceof Task){
					writeNormalAct(tree, child, file, false, "AND");
				}
				else{
					writeNormalAct(tree, child, file, true, "AND");
				}
			}
			
			// for AND, XOR, and VOID we need to have the same constructs, i.e., places between the start activity, and the individual children
			// TODO: Exspect does not properly support deferred choices, so we transform is like a XOR
			if(n instanceof And ||
					n instanceof Xor
					// here is the Exspect specific translation
					|| n instanceof Def
					){
				for(Edge e: ((Block)n).getOutgoingEdges()){
					Node child = e.getTarget();
					if(child instanceof Event){
						// take the child
						// can be multiple event nodes, take transitive closure
						while(child instanceof Event){
							child = ((Event) child).getChildren().get(0);
						}
					}
					String startStatusID = "status_" + startID + "_a_" + getID(e.getTarget());
					String endStatusID = "status" + "_a_" + getID(child) + "_" + endID;
					writeStatus(startStatusID, file);
					Double prob = 1d;
					if(n instanceof Xor
							// here is the Exspect specific translation 
							||n instanceof Def
							){
						
						prob = (Double) e.getIndependentProperty(OutgoingProbability.class);
					}
					writeArc(startID, startStatusID, prob, file);
					writeArc(startStatusID, "a" + getID(child), 1d, file);
					
					writeStatus(endStatusID, file);
					writeArc("a" + getID(child), endStatusID, 1d, file);
					writeArc(endStatusID, endID, prob, file);
				}
			}
			// this translates the LOOPXOR to a protos model, note that we need to add an activity because else we cannot do the data routing
			else if(n instanceof XorLoop
					// TODO: due to Exspect we need to transform LOOPXOR and LOOPDEF in the same way
					|| n instanceof DefLoop
					
					){
				String startStatusID = "status" + startID;
				String midStatusID = "status" + startID + midID;
				String midStatusID1 = "status" + midID + endID + 1;
				String midStatusID2 = "status" + midID + endID + 2;
				String endStatusID = "status" + endID;
				writeStatus(startStatusID, file);
				writeStatus(midStatusID, file);
				writeStatus(midStatusID1, file);
				writeStatus(midStatusID2, file);
				writeStatus(endStatusID, file);
				writeArc(startID, startStatusID, 1d, file);
				writeArc(startStatusID, "a" + getID(((Block)n).getChildren().get(0)), 1d, file);
				writeArc("a" + getID(((Block)n).getChildren().get(0)), midStatusID, 1d, file);
				writeArc(midStatusID, midID, 1d, file);				
				writeArc(midID, midStatusID1, (Double) ((Block)n).getOutgoingEdges().get(1).getIndependentProperty(OutgoingProbability.class), file);
				writeArc(midID, midStatusID2, (Double) ((Block)n).getOutgoingEdges().get(2).getIndependentProperty(OutgoingProbability.class), file);
				writeArc(midStatusID1, "a" + getID(((Block)n).getChildren().get(1)), 1d, file);
				writeArc("a" + getID(((Block)n).getChildren().get(1)), startStatusID, 1d, file);
				writeArc(midStatusID2, "a" + getID(((Block)n).getChildren().get(2)), 1d, file);
				writeArc("a" + getID(((Block)n).getChildren().get(2)), endStatusID, 1d, file);
				writeArc(endStatusID, endID, 1d, file);
			}
			// with the loopdef add a place after the do-block, which denotes a choice between the redo and exit part
			// TODO: also does not work due to Exspect
			// in case of a sequence we need to connect the children horizontal i.e., child 1's output place is child 2's input place etc.
			else if(n instanceof Seq){
				String startStatusID = "status" + startID;
				String endStatusID = "status" + endID;
				writeStatus(startStatusID, file);
				writeStatus(endStatusID, file);
				writeArc(startID, startStatusID, 1d, file);
				Node previous = null;
				for(Node c: ((Seq) n).getChildren()){
					if(previous == null){
						// we have the first activity
						writeArc(startStatusID, "a" + getID(c), 1d, file);
						previous = c;
					}
					else{
						// we have had a previous one
						String tempStatusID = "status" + getID(previous) + getID(c);
						writeStatus(tempStatusID, file);
						writeArc("a" + getID(previous), tempStatusID, 1d, file);
						writeArc(tempStatusID, "a" + getID(c), 1d, file);
						previous = c;
					}
				}
				// we are finished now connect the last activity with the endstatus
				writeArc("a" + getID(previous), endStatusID, 1d, file);
				writeArc(endStatusID, endID, 1d, file);					
			}
			// we have an OR do something smart
			// so the plan is: 
			// - we have a control place for first time execution
			// - a control place for second time execution
			// - 3 silent transitions per subprocess (CHECK)
			else{
				OREncoding(tree, (Or) n, file, startID, specialID, endID);
			}
			// finalise this part of the XML
			file.write("</processflow>\n");
			file.write("<data></data>\n");
			file.write("</subprocess>\n");
			for(Node c: ((Block)n).getChildren()){
				writeNode(tree, c, file);
			}
		}
	}
	
	private void writeNormalAct(ProcessTree tree, Node n, FileWriter file, boolean subprocess, String split) throws IOException, InstantiationException, IllegalAccessException{
		writeAct(tree, n, "", file, subprocess, false, false, false, false, split, "AND");
	}
	
	private void writeAct(ProcessTree tree, Node n, String ID, FileWriter file, boolean subprocess, boolean startprocess, boolean startmodel, boolean endprocess, boolean endmodel, String split, String join) throws IOException, InstantiationException, IllegalAccessException{
		file.write("<flowelement>\n");
		if(n != null){
			file.write("<activity id=\"a" + getID(n) + "\">\n");
			if(subprocess){
				file.write("<name>p" + getID(n) + "</name>\n");
			}
			else{
				file.write("<name>" + n.getName() + "_a_" + getID(n) + "</name>\n");
			}
			
			file.write("<type>");
			if(subprocess){
				file.write("<SubProcessActivity>p" + getID(n) + "</SubProcessActivity>");
			}
			else{
				file.write("<BasicActivity/>");
			}
			file.write("</type>\n");
		}
		else{
			// we have just a start/end activity
			file.write("<activity id=\"" + ID + "\">\n");
			file.write("<name>" + ID + "</name>\n");
			file.write("<type><BasicActivity/></type>\n");
		}
		
		file.write("<startprocess>" + startprocess + "</startprocess>\n");
		file.write("<startmodel>" + startmodel + "</startmodel>\n");
		file.write("<endprocess>" + endprocess + "</endprocess>\n");
		file.write("<endmodel>" + endmodel + "</endmodel>\n");
		file.write("<userinitiative>false</userinitiative>\n");
		file.write("<iteration>single</iteration>\n");
		file.write("<batch_specified>false</batch_specified>\n");
		file.write("<batch_size>0</batch_size>\n");
		file.write("<batch_elapse>0</batch_elapse>\n");
		file.write("<batch_selective>false</batch_selective>\n");
		//role stub
		
		if(n != null && !subprocess && n instanceof Manual){
			file.write("<executor>" + "r" + getID(((Manual)n).getOriginators().iterator().next()) + "</executor>\n");
		}
		else{
			file.write("<executor></executor>\n");
		}
		//end of role stub
		file.write("<sameexecutoroperator>same</sameexecutoroperator>\n");
		file.write("<sameactivitydistributionoperator>same</sameactivitydistributionoperator>\n");
		file.write("<sameresponsibleoperator>same</sameresponsibleoperator>\n");
		
		//simulation stub
		//
		file.write("<simulation_allocate>1</simulation_allocate>\n");
		if(n != null && n.getIndependentProperty(Costs.class) != null){
			file.write("<simulation_cost>" + toStringObject(n.getIndependentProperty(Costs.class)) + "</simulation_cost>\n");
		}
		else{
			//file.write("<simulation_cost>" + toStringObject(Costs.class.newInstance().getDefaultValue()) + "</simulation_cost>\n");
			file.write("<simulation_cost>" + toStringObject(1) + "</simulation_cost>\n");
		}
		file.write("<simulation_frequency>1.000000e0</simulation_frequency>\n");
		//file.write("<duration><Constant mean=\"1.000000e0\"/></duration>\n");
		file.write("<duration><");
		writeDistribution(tree, n, file);
		file.write("/></duration>\n");
		
		file.write("<priority><Constant mean=\"1.000000e0\"/></priority>\n");
		//
		//end simulation stub
		
		file.write("<analysis_in>" + join + "</analysis_in>\n");
		file.write("<analysis_out>" + split + "</analysis_out>\n");
		file.write("<description></description>\n");
		file.write("<workinstruction></workinstruction>\n");
		file.write("</activity>\n");
		file.write("</flowelement>\n");
	}
	
	private void writeOptions(ProcessTree tree, FileWriter file) throws IOException, InstantiationException, IllegalAccessException{
		file.write(startText());
		// fill with simulation things
		SimulationProperty simProp = (SimulationProperty) tree.getIndependentProperty(SimulationProperty.class);
		if(simProp == null){
			simProp = SimulationProperty.class.newInstance().getDefaultValue();
		}
		file.write("<BatchSize>1</BatchSize>\n");
		file.write("<NumberOfBatches>1000000</NumberOfBatches>\n");
		file.write("<NumberOfRuns>" + simProp.getNrSubRuns() + "</NumberOfRuns>\n");
		file.write("<RunLength>" + simProp.getLengthOfSubRuns() + "</RunLength>\n");
		file.write("<NumberOfStartRuns>0</NumberOfStartRuns>\n");
		file.write("<SimulationArrival><");
		writeDistribution(tree, file);
		file.write("/></SimulationArrival>\n");
		// end of fill with simulation things
		file.write(endText());
	}
	// TODO: this is a big hack w.r.t. probability but there is no other way...
	private void writeArc(String from, String to, Double probability, FileWriter file) throws IOException{
		file.write("<processarc from=\"" + from + "\" to=\"" + to + "\">\n");
		file.write("<type>Action</type>\n");
		file.write("<label></label>\n");
		file.write("<description></description>\n");
		file.write("<workinstruction></workinstruction>\n");
		file.write("<sim_frequency>" + toString(probability*10000000) + "</sim_frequency>\n");
		file.write("</processarc>\n");
	}
	
	private void writeStatus(String ID, FileWriter file) throws IOException{
		file.write("<flowelement>\n");
		file.write("<status id=\"" + ID + "\">\n");
		file.write("<name>" + ID + "</name>\n");
		file.write("<description></description>\n");
		file.write("<workinstruction></workinstruction>\n");
		file.write("</status>\n");
		file.write("</flowelement>\n");
	}
	
	private String toStringObject(Object argument){
		if(argument instanceof Integer){
			return toString(((Integer) argument).intValue());
		}
		else if(argument instanceof Float){
			return toString(((Float) argument).floatValue());
		}
		else if(argument instanceof Double){
			return toString((Double)argument);
		}
		else{
			return "";
		}
	}
	
	private String toString(double argument){
		DecimalFormat protosFormatter = new DecimalFormat("0.000000E0");
		return protosFormatter.format(argument).replace(",", ".").replace("E", "e");
	}
	
	private void writeDistribution(ProcessTree tree, FileWriter file) throws InstantiationException, IllegalAccessException, IOException{
		// we need to get the arrival process
		Distribution distr = (Distribution) tree.getIndependentProperty(ArrivalProcess.class);
		if(distr == null){
			distr = Distribution.class.newInstance().getDefaultValue();
		}
		DistributionType type = distr.distributionType;
		if(type == DistributionType.NegativeExponential){
			file.write("Nexp");
		}
		if(type == DistributionType.Beta){
			file.write("Beta");
		}
		if(type == DistributionType.Normal){
			file.write("Normal");
		}
		if(type == DistributionType.Erlang){
			file.write("Erlang");
		}
		if(type == DistributionType.Gamma){
			file.write("Gamma");
		}
		if(type == DistributionType.Uniform){
			file.write("Uniform");
		}
		if(type == DistributionType.Constant){
			file.write("Constant");
		}
		file.write(" ");
		
		if(type == DistributionType.NegativeExponential || type == DistributionType.Beta || type == DistributionType.Normal || 
				type == DistributionType.Erlang || type == DistributionType.Gamma || type == DistributionType.Constant){
			file.write("mean=\"" + toStringObject(distr.mean) + "\" ");
		}
		if(type == DistributionType.Normal || type == DistributionType.Gamma){
			file.write("variance=\"" + toStringObject(distr.variance) + "\" ");
		}
		if(type == DistributionType.Beta || type == DistributionType.Uniform){
			file.write("minimum=\"" + toStringObject(distr.minimum) + "\" ");
		}
		if(type == DistributionType.Beta || type == DistributionType.Uniform){
			file.write("maximum=\"" + toStringObject(distr.maximum) + "\" ");
		}				 
		if(type == DistributionType.Erlang){
			file.write("number=\"" + toStringObject(distr.number) + "\" ");
		}
	}

	private void writeDistribution(ProcessTree tree, Node n, FileWriter file) throws InstantiationException, IllegalAccessException, IOException{
		// we need to get the Yerkes Dodson and convert it
		TaskDistribution distr = null;
		if(n != null && n.getIndependentProperty(TaskDistribution.class) != null){
			distr = (TaskDistribution) n.getIndependentProperty(TaskDistribution.class);
		}
		else if(n != null && n.getIndependentProperty(YerkesDodsonInterval.class) != null){
			distr = YerkesDodsonInterval2TaskDistribution.convert(n.getIndependentProperty(YerkesDodsonInterval.class));
		}
		else{
			// simply because Protos is too sensitive else
			distr = new TaskDistribution(new Distribution(DistributionType.Constant, 0d, null, null, null, null));
		}
		DistributionType type = distr.distributionType;
		if(type == DistributionType.NegativeExponential){
			file.write("Nexp");
		}
		if(type == DistributionType.Beta){
			file.write("Beta");
		}
		if(type == DistributionType.Normal){
			file.write("Normal");
		}
		if(type == DistributionType.Erlang){
			file.write("Erlang");
		}
		if(type == DistributionType.Gamma){
			file.write("Gamma");
		}
		if(type == DistributionType.Uniform){
			file.write("Uniform");
		}
		if(type == DistributionType.Constant){
			file.write("Constant");
		}
		file.write(" ");
		
		if(type == DistributionType.NegativeExponential || type == DistributionType.Beta || type == DistributionType.Normal || 
				type == DistributionType.Erlang || type == DistributionType.Gamma || type == DistributionType.Constant){
			file.write("mean=\"" + toStringObject(distr.mean) + "\" ");
		}
		if(type == DistributionType.Normal || type == DistributionType.Gamma){
			file.write("variance=\"" + toStringObject(distr.variance) + "\" ");
		}
		if(type == DistributionType.Beta || type == DistributionType.Uniform){
			file.write("minimum=\"" + toStringObject(distr.minimum) + "\" ");
		}
		if(type == DistributionType.Beta || type == DistributionType.Uniform){
			file.write("maximum=\"" + toStringObject(distr.maximum) + "\" ");
		}				 
		if(type == DistributionType.Erlang){
			file.write("number=\"" + toStringObject(distr.number) + "\" ");
		}
	}
	
	private void OREncoding(ProcessTree tree, Or n, FileWriter file, String startID, String specialID, String endID) throws IOException, InstantiationException, IllegalAccessException{
		// This is the encoding of the OR using the last child as being the default choice
		String firstTimeID = "statusFirst" + getID(n);
		String secondTimeID = "statusSecond" + getID(n);
		writeStatus(firstTimeID, file);
		writeStatus(secondTimeID, file);
		writeArc(startID, firstTimeID, 1d, file);
		writeArc(secondTimeID, endID, 1d, file);
		
		OROptions orOption = (OROptions) n.getIndependentProperty(ORSemantic.class);
		if(orOption == null){
			orOption = ORSemantic.class.newInstance().getDefaultValue();
		}
		
		for(Edge e: n.getOutgoingEdges()){
			// we have one of the subprocesses, so make the init place, the mid place, the end place
			String statusStart = "statusStart" + getID(e.getTarget());
			String statusDo = "statusDo" + getID(e.getTarget());
			String statusSkip = "statusSkip" + getID(e.getTarget());
			String statusMid = "statusMid" + getID(e.getTarget());
			String statusEnd = "statusEnd" + getID(e.getTarget());
			String tau1 = "tau1" + getID(e.getTarget());
			String tau2 = "tau2" + getID(e.getTarget());
			String tau3 = "tau3" + getID(e.getTarget());
			String tau4 = "tau4" + getID(e.getTarget());
			writeStatus(statusStart, file);
			writeStatus(statusDo, file);
			writeStatus(statusSkip, file);
			writeStatus(statusMid, file);
			writeStatus(statusEnd, file);
			
			writeArc(startID, statusStart, 1d, file);
			writeArc(statusEnd, endID, 1d, file);
			Double probDo = (Double) e.getIndependentProperty(OutgoingProbability.class);
			// Only do this as default option
			if(orOption.equals(OROptions.Default) && n.getOutgoingEdges().indexOf(e) == n.getOutgoingEdges().size() - 1){
				probDo = 0d;
			}
			Double probSkip = 1 - probDo;
			// Tau 1 (split into Do and Skip time)
			writeArc(statusStart, tau1, 1d, file);
			writeArc(tau1, statusDo, probDo, file);
			writeArc(tau1, statusSkip, probSkip, file);
			
			// Tau 2 (first time)
			writeArc(statusDo, tau2, 1d, file);
			writeArc(tau2, statusMid, 1d, file);
			writeArc(firstTimeID, tau2, 1d, file);
			writeArc(tau2, secondTimeID, 1d, file);
			
			// Tau 3 (second time)
			writeArc(statusDo, tau3, 1d, file);
			writeArc(tau3, statusMid, 1d, file);
			writeArc(secondTimeID, tau3, 1d, file);
			writeArc(tau3, secondTimeID, 1d, file);
			
			// Tau 4 (skip)
			writeArc(statusSkip, tau4, 1d, file);
			writeArc(tau4, statusEnd, 1d, file);
			// only do this when scaling
			if(orOption.equals(OROptions.Scaling)){
				writeArc(secondTimeID, tau4, 1d, file);
				writeArc(tau4, secondTimeID, 1d, file);
			}
			
			// Activity
			writeArc(statusMid, "a" + getID(e.getTarget()), 1d, file);
			writeArc("a" + getID(e.getTarget()), statusEnd, 1d, file);
			// also do this when scaling
			if(orOption.equals(OROptions.Scaling)){
				// we have to add all "reset" edges
				writeArc(statusSkip, specialID, 1d, file);
				writeArc(specialID, statusStart, 1d, file);
			}
			// only do this when not scaling
			if(!orOption.equals(OROptions.Scaling)){
				// Add arc between status End and Default transition
				// only if it is not the last transition
				if(n.getOutgoingEdges().indexOf(e) != n.getOutgoingEdges().size() - 1){
					writeArc(statusEnd, specialID, 1d, file);
					writeArc(specialID, statusEnd, 1d, file);
				}
				if(n.getOutgoingEdges().indexOf(e) == n.getOutgoingEdges().size() - 1){
					// we have the last element, so add the remaining edges
					// tau 4
					// we need to add this s.t. it cannot be skipped unless some of the others have executed
					writeArc(secondTimeID, tau4, 1d, file);
					writeArc(tau4, secondTimeID, 1d, file);
					
					// add the other options to Special
					writeArc(statusSkip, specialID, 1d, file);
					writeArc(firstTimeID, specialID, 1d, file);
					writeArc(specialID, secondTimeID, 1d, file);
					writeArc(specialID, statusMid, 1d, file);
				}
			}
		}
	}
	
	private int getID(ProcessTreeElement elem){
		int hash = elem.getID().hashCode();
		if(hash < 0){
			hash = Math.abs(hash);
		}
		return hash; 
	}
	
	private String startText(){
		return "<metanames></metanames>\n" +
		"<protosmodeloptions>\n" +
		"<HandleWidth>120</HandleWidth>\n" +
		"<HandleHeight>30</HandleHeight>\n" +
		"<RoleRouteColumnSep>150</RoleRouteColumnSep>\n" +
		"<DistributionRouteColumnSep>150</DistributionRouteColumnSep>\n" +
		"<DataRouteColumnSep>150</DataRouteColumnSep>\n" +
		"<ApplicationRouteColumnSep>150</ApplicationRouteColumnSep>\n" +
		"<RoleRouteRowSep>50</RoleRouteRowSep>\n" +
		"<DistributionRouteRowSep>50</DistributionRouteRowSep>\n" +
		"<DataRouteRowSep>50</DataRouteRowSep>\n" +
		"<ApplicationRouteRowSep>50</ApplicationRouteRowSep>\n" +
		"<RoleRouteBlockSize>40</RoleRouteBlockSize>\n" +
		"<DistributionRouteBlockSize>40</DistributionRouteBlockSize>\n" +
		"<DataRouteBlockSize>40</DataRouteBlockSize>\n" +
		"<ApplicationRouteBlockSize>40</ApplicationRouteBlockSize>\n" +
		"<Grid>15</Grid>\n" +
		"<Snapping>1</Snapping>\n" +
		"<GridVisible>1</GridVisible>\n" +
		"<Font><name>-11 0 0 0 400 0 0 0 0 3 2 1 34 Arial&#009;0</name>\n" +
		"<size>11</size>\n" +
		"<underline>false</underline>\n" +
		"<bold>false</bold>\n" +
		"<italic>false</italic>\n" +
		"<color>0</color>\n" +
		"</Font>\n";
	}
	
	private String endText(){
		return "<Description></Description>\n" +
		"<Workinstruction></Workinstruction>\n" +
		"<ExportTitlePage>true</ExportTitlePage>\n" +
		"<ExportHeader>true</ExportHeader>\n" +
		"<ExportFooter>true</ExportFooter>\n" +
		"<ExportTableOfContents>true</ExportTableOfContents>\n" +
		"<ExportDescription>true</ExportDescription>\n" +
		"<ExportWorkflowWorkinstruction>true</ExportWorkflowWorkinstruction>\n" +
		"<ExportOverview>true</ExportOverview>\n" +
		"<ExportOverviewFigure>true</ExportOverviewFigure>\n" +
		"<ExportOverviewTable>true</ExportOverviewTable>\n" +
		"<ExportRoles>true</ExportRoles>\n" +
		"<ExportRolesFigure>true</ExportRolesFigure>\n" +
		"<ExportRolesTable>true</ExportRolesTable>\n" +
		"<ExportDistributions>true</ExportDistributions>\n" +
		"<ExportGroups>true</ExportGroups>\n" +
		"<ExportAnalysis>true</ExportAnalysis>\n" +
		"<ExportAnalysisRoleRoute>true</ExportAnalysisRoleRoute>\n" +
		"<ExportAnalysisRoleTable>true</ExportAnalysisRoleTable>\n" +
		"<ExportAnalysisDistributionRoute>true</ExportAnalysisDistributionRoute>\n" +
		"<ExportAnalysisDistributionTable>true</ExportAnalysisDistributionTable>\n" +
		"<ExportAnalysisDataRoute>true</ExportAnalysisDataRoute>\n" +
		"<ExportAnalysisApplicationRoute>true</ExportAnalysisApplicationRoute>\n" +
		"<ExportProcess>true</ExportProcess>\n" +
		"<ExportProcessDescriptionInstruction>0</ExportProcessDescriptionInstruction>\n" +
		"<ExportProcessDescriptionInstruction2>0</ExportProcessDescriptionInstruction2>\n" +
		"<ExportProcessFigure>true</ExportProcessFigure>\n" +
		"<ExportProcessContents>true</ExportProcessContents>\n" +
		"<ExportProcessActivities>true</ExportProcessActivities>\n" +
		"<ExportProcessTriggers>true</ExportProcessTriggers>\n" +
		"<ExportProcessStatus>true</ExportProcessStatus>\n" +
		"<ExportProcessBuffers>true</ExportProcessBuffers>\n" +
		"<ExportProcessArcs>true</ExportProcessArcs>\n" +
		"<ExportProcessApplications>true</ExportProcessApplications>\n" +
		"<ExportProcessData>true</ExportProcessData>\n" +
		"<ExportMeta>true</ExportMeta>\n" +
		"<ExportSuppressEmpty>true</ExportSuppressEmpty>\n" +
		"<ExportActivityWorkinstruction>true</ExportActivityWorkinstruction>\n" +
		"<ExportActivityDescription>true</ExportActivityDescription>\n" +
		"<ExportActivityRole>true</ExportActivityRole>\n" +
		"<ExportActivityResponsible>true</ExportActivityResponsible>\n" +
		"<ExportActivityDistribution>true</ExportActivityDistribution>\n" +
		"<ExportActivityBatch>true</ExportActivityBatch>\n" +
		"<ExportActivityData>true</ExportActivityData>\n" +
		"<ExportActivityArc>0</ExportActivityArc>\n" +
		"<ExportActivityApplication>true</ExportActivityApplication>\n" +
		"<ExportRoleDescription>true</ExportRoleDescription>\n" +
		"<ExportRoleWorkinstruction>true</ExportRoleWorkinstruction>\n" +
		"<ExportDistributionDescription>true</ExportDistributionDescription>\n" +
		"<ExportDistributionWorkinstruction>true</ExportDistributionWorkinstruction>\n" +
		"<ExportGroupDescription>true</ExportGroupDescription>\n" +
		"<ExportGroupWorkinstruction>true</ExportGroupWorkinstruction>\n" +
		"<ExportTriggerDescription>true</ExportTriggerDescription>\n" +
		"<ExportTriggerWorkinstruction>true</ExportTriggerWorkinstruction>\n" +
		"<ExportStatusDescription>true</ExportStatusDescription>\n" +
		"<ExportStatusWorkinstruction>true</ExportStatusWorkinstruction>\n" +
		"<ExportBufferDescription>true</ExportBufferDescription>\n" +
		"<ExportBufferWorkinstruction>true</ExportBufferWorkinstruction>\n" +
		"<ExportArcDescription>true</ExportArcDescription>\n" +
		"<ExportArcWorkinstruction>true</ExportArcWorkinstruction>\n" +
		"<ExportApplicationDescription>true</ExportApplicationDescription>\n" +
		"<ExportApplicationWorkinstruction>true</ExportApplicationWorkinstruction>\n" +
		"<ExportDataDescription>true</ExportDataDescription>\n" +
		"<ExportDataWorkinstruction>true</ExportDataWorkinstruction>\n" +
		"<ExportKeepRows>true</ExportKeepRows>\n" +
		"<ExportSimulation>true</ExportSimulation>\n" +
		"<ExportActivityPart>true</ExportActivityPart>\n" +
		"<ExportTriggerPart>true</ExportTriggerPart>\n" +
		"<ExportApplicationPart>true</ExportApplicationPart>\n" +
		"<ExportDataPart>true</ExportDataPart>\n" +
		"<ExportColorPrinter>false</ExportColorPrinter>\n" +
		"<ExportRoleTableContents>0</ExportRoleTableContents>\n" +
		"<ExportDistributionTableContents>0</ExportDistributionTableContents>\n" +
		"<ExportTableSeparation>1.000000e0</ExportTableSeparation>\n" +
		"<ExportWordProcessor>0</ExportWordProcessor>\n" +
		"<ExportLayout>0</ExportLayout>\n" +
		"<HtmlMail>false</HtmlMail>\n" +
		"<HtmlMailAddress></HtmlMailAddress>\n" +
		"<HtmlBaseDirectory></HtmlBaseDirectory>\n" +
		"<HtmlEnvironment>false</HtmlEnvironment>\n" +
		"<HtmlHomePage></HtmlHomePage>\n" +
		"<HtmlInstructionTop>true</HtmlInstructionTop>\n" +
		"<HtmlHyperlinks>false</HtmlHyperlinks>\n" +
		"<HtmlStartBrowser>true</HtmlStartBrowser>\n" +
		"<HtmlOrientationLeft>true</HtmlOrientationLeft>\n" +
		"<HtmlMaximize>true</HtmlMaximize>\n" +
		"<HtmlMaximizeCurrentWindow>true</HtmlMaximizeCurrentWindow>\n" +
		"<HtmlActivity>true</HtmlActivity>\n" +
		"<HtmlActivityApplications>true</HtmlActivityApplications>\n" +
		"<HtmlActivityData>true</HtmlActivityData>\n" +
		"<HtmlActivityDescription>true</HtmlActivityDescription>\n" +
		"<HtmlActivityMeta>true</HtmlActivityMeta>\n" +
		"<HtmlActivityInstruction>true</HtmlActivityInstruction>\n" +
		"<HtmlActivityInvolved>true</HtmlActivityInvolved>\n" +
		"<HtmlActivityResponsible>true</HtmlActivityResponsible>\n" +
		"<HtmlActivityRole>true</HtmlActivityRole>\n" +
		"<HtmlActivitySimulation>true</HtmlActivitySimulation>\n" +
		"<HtmlActivityDistribution>true</HtmlActivityDistribution>\n" +
		"<HtmlApplication>true</HtmlApplication>\n" +
		"<HtmlApplicationData>true</HtmlApplicationData>\n" +
		"<HtmlApplicationDescription>true</HtmlApplicationDescription>\n" +
		"<HtmlApplicationMeta>true</HtmlApplicationMeta>\n" +
		"<HtmlApplicationInstruction>true</HtmlApplicationInstruction>\n" +
		"<HtmlApplicationInvolved>true</HtmlApplicationInvolved>\n" +
		"<HtmlBuffer>true</HtmlBuffer>\n" +
		"<HtmlBufferDescription>true</HtmlBufferDescription>\n" +
		"<HtmlBufferMeta>true</HtmlBufferMeta>\n" +
		"<HtmlBufferInstruction>true</HtmlBufferInstruction>\n" +
		"<HtmlConnection>true</HtmlConnection>\n" +
		"<HtmlConnectionDescription>true</HtmlConnectionDescription>\n" +
		"<HtmlConnectionMeta>true</HtmlConnectionMeta>\n" +
		"<HtmlConnectionInstruction>true</HtmlConnectionInstruction>\n" +
		"<HtmlConnectionData>true</HtmlConnectionData>\n" +
		"<HtmlData>true</HtmlData>\n" +
		"<HtmlDataData>true</HtmlDataData>\n" +
		"<HtmlDataDescription>true</HtmlDataDescription>\n" +
		"<HtmlDataMeta>true</HtmlDataMeta>\n" +
		"<HtmlDataInstruction>true</HtmlDataInstruction>\n" +
		"<HtmlDataInvolved>true</HtmlDataInvolved>\n" +
		"<HtmlGroup>true</HtmlGroup>\n" +
		"<HtmlGroupData>true</HtmlGroupData>\n" +
		"<HtmlGroupDescription>true</HtmlGroupDescription>\n" +
		"<HtmlGroupMeta>true</HtmlGroupMeta>\n" +
		"<HtmlGroupInstruction>true</HtmlGroupInstruction>\n" +
		"<HtmlProcessModel>true</HtmlProcessModel>\n" +
		"<HtmlProcessModelDescription>true</HtmlProcessModelDescription>\n" +
		"<HtmlProcessModelMeta>true</HtmlProcessModelMeta>\n" +
		"<HtmlProcessModelInstruction>true</HtmlProcessModelInstruction>\n" +
		"<HtmlRole>true</HtmlRole>\n" +
		"<HtmlRoleDescription>true</HtmlRoleDescription>\n" +
		"<HtmlRoleMeta>true</HtmlRoleMeta>\n" +
		"<HtmlRoleInstruction>true</HtmlRoleInstruction>\n" +
		"<HtmlStatus>true</HtmlStatus>\n" +
		"<HtmlStatusDescription>true</HtmlStatusDescription>\n" +
		"<HtmlStatusMeta>true</HtmlStatusMeta>\n" +
		"<HtmlStatusInstruction>true</HtmlStatusInstruction>\n" +
		"<HtmlSubprocess>true</HtmlSubprocess>\n" +
		"<HtmlSubprocessApplications>true</HtmlSubprocessApplications>\n" +
		"<HtmlSubprocessData>true</HtmlSubprocessData>\n" +
		"<HtmlSubprocessDescription>true</HtmlSubprocessDescription>\n" +
		"<HtmlSubprocessMeta>true</HtmlSubprocessMeta>\n" +
		"<HtmlSubprocessInstruction>true</HtmlSubprocessInstruction>\n" +
		"<HtmlSubprocessInvolved>true</HtmlSubprocessInvolved>\n" +
		"<HtmlSubprocessResponsible>true</HtmlSubprocessResponsible>\n" +
		"<HtmlDistribution>true</HtmlDistribution>\n" +
		"<HtmlDistributionData>true</HtmlDistributionData>\n" +
		"<HtmlDistributionDescription>true</HtmlDistributionDescription>\n" +
		"<HtmlDistributionMeta>true</HtmlDistributionMeta>\n" +
		"<HtmlDistributionInstruction>true</HtmlDistributionInstruction>\n" +
		"<HtmlTrigger>true</HtmlTrigger>\n" +
		"<HtmlTriggerApplications>true</HtmlTriggerApplications>\n" +
		"<HtmlTriggerData>true</HtmlTriggerData>\n" +
		"<HtmlTriggerDescription>true</HtmlTriggerDescription>\n" +
		"<HtmlTriggerMeta>true</HtmlTriggerMeta>\n" +
		"<HtmlTriggerInstruction>true</HtmlTriggerInstruction>\n" +
		"<HtmlTriggerInvolved>true</HtmlTriggerInvolved>\n" +
		"<HtmlTriggerSimulation>true</HtmlTriggerSimulation>\n" +
		"<ExportWorkflowHistory>true</ExportWorkflowHistory>\n" +
		"<HtmlWeb1>false</HtmlWeb1>\n" +
		"<HtmlWeb1String></HtmlWeb1String>\n" +
		"<HtmlWeb2>false</HtmlWeb2>\n" +
		"<HtmlWeb2String></HtmlWeb2String>\n" +
		"<ExportStartWordProcessor>true</ExportStartWordProcessor>\n" +
		"<ExportLinkModel>true</ExportLinkModel>\n" +
		"<ExportRoleTable>false</ExportRoleTable>\n" +
		"<ExportDistributionTable>false</ExportDistributionTable>\n" +
		"<ExportDataRoute>false</ExportDataRoute>\n" +
		"<ExportApplicationRoute>false</ExportApplicationRoute>\n" +
		"<ResponsibleRouteColumnSep>150</ResponsibleRouteColumnSep>\n" +
		"<ResponsibleRouteRowSep>50</ResponsibleRouteRowSep>\n" +
		"<ResponsibleRouteBlockSize>40</ResponsibleRouteBlockSize>\n" +
		"<ExportResponsibleRoute>true</ExportResponsibleRoute>\n" +
		"<ExportResponsibleTable>true</ExportResponsibleTable>\n" +
		"<ExportProcessResponsibleTable>false</ExportProcessResponsibleTable>\n" +
		"<ResponsibleTableContents>0</ResponsibleTableContents>\n" +
		"<WebSearch>false</WebSearch>\n" +
		"<WebSearchString></WebSearchString>\n" +
		"<WebRegistration>false</WebRegistration>\n" +
		"<WebRegistrationString></WebRegistrationString>\n" +
		"<ExportActivityAssignment>true</ExportActivityAssignment>\n" +
		"<ShowDrawing>true</ShowDrawing>\n" +
		"<ExportRtfDrawing>true</ExportRtfDrawing>\n" +
		"<ExportHtmlDrawing>true</ExportHtmlDrawing>\n" +
		"<ExportAnalysisRACITable>true</ExportAnalysisRACITable>\n" +
		"<ExportRACITableContents>0</ExportRACITableContents>\n" +
		"<ExportHtmlSuppressEmpty>false</ExportHtmlSuppressEmpty>\n" +
		"<ExportHtmlExecutorAnalysis>0</ExportHtmlExecutorAnalysis>\n" +
		"<ExportHtmlResponsibleAnalysis>0</ExportHtmlResponsibleAnalysis>\n" +
		"<ExportHtmlDistributionAnalysis>0</ExportHtmlDistributionAnalysis>\n" +
		"<ExportHtmlDataAnalysis>0</ExportHtmlDataAnalysis>\n" +
		"<ExportHtmlApplicationAnalysis>0</ExportHtmlApplicationAnalysis>\n" +
		"<ExportHtmlRACIAnalysis>0</ExportHtmlRACIAnalysis>\n" +
		"<ExportHtmlHistory>0</ExportHtmlHistory>\n" +
		"<ProtosVersion>8.0.2</ProtosVersion>\n" +
		"</protosmodeloptions>\n";
	}
}
