package org.processmining.processtree.editor.editorpanel;

import java.awt.event.ActionEvent;
import java.util.HashSet;
import java.util.UUID;

import javax.swing.JSlider;
import javax.swing.JTextField;

import org.processmining.processtree.Block;
import org.processmining.processtree.Node;
import org.processmining.processtree.Originator;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.ProcessTree.Type;
import org.processmining.processtree.editor.graphpanel.GraphPanel;
import org.processmining.processtree.editor.interaction.GraphMouseListener;
import org.processmining.processtree.impl.AbstractBlock;
import org.processmining.processtree.impl.AbstractEvent;
import org.processmining.processtree.impl.AbstractTask;

public class EditorActionListenerAddNode implements EditorActionListener {
	ProcessTree tree;
	GraphMouseListener graphMouseListener;
	GraphPanel graphPanel;
	Type nodeType;
	JTextField nameField;
	JSlider childIndex;
	
	
	public EditorActionListenerAddNode(ProcessTree tree, GraphMouseListener graphMouseListener, GraphPanel graphPanel, Type nodeType, JTextField nameField, JSlider childIndex){
		this.tree = tree;
		this.graphMouseListener = graphMouseListener;
		this.graphPanel = graphPanel;
		this.nodeType = nodeType;
		this.nameField = nameField;
		this.childIndex = childIndex;
	}
	
	public void actionPerformed(ActionEvent e) {
		if(!isEnabled()){
			return;
		}
		UUID elementId = graphMouseListener.getSelectedID();
		// case distinction on the type of the nodes
		Node n;
		//String name = nameField.getText();
		String name = "";
		String message = "";
		switch(nodeType){
			case SEQ:
				n = new AbstractBlock.Seq(name);
				break;
			case AND:
				n = new AbstractBlock.And(name);
				break;
			case XOR:
				n = new AbstractBlock.Xor(name);
				break;
			case DEF:
				n = new AbstractBlock.Def(name);
				break;
			case OR:
				n = new AbstractBlock.Or(name);
				break;
			case LOOPXOR:
				n = new AbstractBlock.XorLoop(name);
				break;
			case LOOPDEF:
				n = new AbstractBlock.DefLoop(name);
				break;
			case PLACEHOLDER:
				n = new AbstractBlock.PlaceHolder(name);
				break;
			case TIMEOUT:
				n = new AbstractEvent.TimeOut(name, message);
				break;
			case MESSAGE:
				n = new AbstractEvent.Message(name, message);
				break;
			case MANTASK:
				n = new AbstractTask.Manual(name, new HashSet<Originator>());
				break;
			default:
				n = new AbstractTask.Automatic(name);
				break;
		}
		n.setProcessTree(tree);
		tree.addNode(n);
		if(tree.getRoot() == null){
			// this becomes the root
			tree.setRoot(n);
		}
		else{
			if(childIndex.isEnabled() && childIndex.getValue() < ((Block)tree.getElement(elementId)).getChildren().size()){
				tree.addEdge(((Block)tree.getElement(elementId)).addChildAt(n, childIndex.getValue()));
			}
			else{
				tree.addEdge(((Block)tree.getElement(elementId)).addChild(n));
			}
		}
		nameField.setText("");
		graphPanel.redraw();		
	}
	
	public boolean isEnabled(){
		UUID elementId = graphMouseListener.getSelectedID();
		if(!(tree.getElement(elementId) instanceof Block) && (tree.getRoot() != null)){
			return false;
		}
		return true;
	}


}
