package org.processmining.processtree.editor.editorpanel;

import info.clearthought.layout.TableLayout;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.UUID;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;

import org.jgraph.JGraph;
import org.processmining.processtree.Block;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Node;
import org.processmining.processtree.Originator;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.ProcessTree.Type;
import org.processmining.processtree.ProcessTreeElement;
import org.processmining.processtree.editor.graphpanel.GraphPanel;
import org.processmining.processtree.editor.interaction.GraphMouseListener;
import org.processmining.processtree.impl.AbstractBlock;
import org.processmining.processtree.impl.AbstractEvent;
import org.processmining.processtree.impl.AbstractTask;
import org.processmining.processtree.impl.ProcessTreeImpl;
import org.processmining.processtree.visualization.tree.TreeLayoutBuilder;

public class EditorPanelControlFlow {
	ProcessTree tree;
	int maxWidth;
	int maxHeight;
	GraphMouseListener graphMouseListener;
	GraphPanel graphPanel;
	JSlider childIndex;
	Collection<JComponent> components;
	JTextField nameField;
	JTextField edgeTextField;
	JComboBox<Type> typeCombo;
	private int rows = 10;
	private int heightOfRows = 25;
	private int heightOfOperators = 30;
	private Collection<JButton> localComponents;
	
	
	public EditorPanelControlFlow(ProcessTree tree, int maxWidth, int maxHeight, GraphMouseListener graphMouseListener, GraphPanel graphPanel, Collection<JComponent> components){
		this.tree = tree;
		this.maxWidth = maxWidth;
		this.maxHeight = maxHeight;
		this.graphMouseListener = graphMouseListener;
		this.graphPanel = graphPanel;
		this.components = components;
		this.localComponents = new HashSet<JButton>();
		this.childIndex = new JSlider();
	}
	
	public int getHeight(){
		// return the height in terms of elements
		// added some for the border
		return ((rows - 3) * heightOfRows) + (3 * heightOfOperators) + 24;
	}
	
	public JComponent createControlFlowPerspective(){
		int columns = 8;
		// scaling for the border
		int width = (maxWidth - 8) / columns;
		double[][] layout = new double[2][Math.max(columns, rows)];
		for(int i = 0; i < columns; i++){
			layout[0][i] = width;
		}
		for(int i = 0; i < rows; i++){
			layout[1][i] = heightOfRows;
			layout[1][4] = heightOfOperators;
			layout[1][5] = heightOfOperators;
			layout[1][6] = heightOfOperators;
		}		
		JPanel panel = new JPanel();
		panel.setLayout(new TableLayout(layout));
		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Control-Flow Perspective"), BorderFactory.createBevelBorder(BevelBorder.LOWERED)));
		
		int offset = 0;
		JLabel nodeNameLabel = new JLabel("Name:");
		nodeNameLabel.setPreferredSize(new Dimension(2 * width, heightOfRows));
		nameField = new JTextField();
		nameField.setPreferredSize(new Dimension(6 * width, heightOfRows));
		List<Class<? extends ProcessTreeElement>> validElements = new ArrayList<>();
		validElements.add(Node.class);
		validElements.add(Edge.class);
		validElements.add(ProcessTree.class);
		nameField.addKeyListener(new EditorKeyListenerNameField(tree, graphMouseListener, graphPanel, nameField, validElements));
		panel.add(nodeNameLabel, "0, " + offset + ", 1, " + offset + ", l, c");
		panel.add(nameField, "2, " + offset + ", 7, " + offset + ", l, c");
		offset += 1;
		createChildSlider(panel, offset, 0, width, heightOfRows);
		offset += 1;
		createLeftRightButton(panel, offset, 0, width, heightOfRows);
		offset += 2;
		createAddButton(panel, Type.MANTASK, offset, 3, width, heightOfOperators);
		createAddButton(panel, Type.AUTOTASK, offset, 4, width, heightOfOperators);
		offset += 1;
		createAddButton(panel, Type.SEQ, offset, 0, width, heightOfOperators);
		createAddButton(panel, Type.AND, offset, 1, width, heightOfOperators);
		createAddButton(panel, Type.XOR, offset, 2, width, heightOfOperators);
		createAddButton(panel, Type.DEF, offset, 3, width, heightOfOperators);
		offset += 0;
		createAddButton(panel, Type.OR, offset, 4, width, heightOfOperators);
		createAddButton(panel, Type.LOOPXOR, offset, 5, width, heightOfOperators);
		createAddButton(panel, Type.LOOPDEF, offset, 6, width, heightOfOperators);
		createAddButton(panel, Type.PLACEHOLDER, offset, 7, width, heightOfOperators);
		offset += 1;
		createAddButton(panel, Type.TIMEOUT, offset, 0, width, heightOfOperators);
		createAddButton(panel, Type.MESSAGE, offset, 1, width, heightOfOperators);
		createOperatorChanger(panel, offset, 0, width, heightOfOperators);
		offset += 1;
		createAddEdgeButton(panel, offset, 0, width, heightOfRows);
		offset += 1;
		createBlockingHidingToggleButtons(panel, offset, 0, width, heightOfRows);
		offset += 1;
		createDeleteButton(panel, offset, 2, width, heightOfRows);
		update();
		return panel;
	}
	
	public void update(){
		updateSlider();
		updateNameField();
		updateEdgeTextField();
		updateTypeComboBox();
		checkEnablement();
	}
	
	private void updateSlider(){
		UUID selectedID = graphMouseListener.getSelectedID();
		if(selectedID != null){
			if(tree.getElement(selectedID) == null){
				childIndex.setEnabled(false);
			}
			if(!(tree.getElement(selectedID) instanceof Block)){
				childIndex.setEnabled(false);
			}
			else{
				if(((Block)tree.getElement(selectedID)).getChildren().size() == 0){
					childIndex.setValue(0);
					childIndex.setEnabled(false);
					return;
				}
				childIndex.setEnabled(true);
				childIndex.setMinimum(0);
				childIndex.setMaximum(((Block)tree.getElement(selectedID)).getChildren().size());
				childIndex.setValue(childIndex.getMaximum());
			}
		}
	}
	
	private void updateNameField(){
		UUID selectedID = graphMouseListener.getSelectedID();
		if(tree.getElement(selectedID) != null && (tree.getElement(selectedID) instanceof Node ||
				tree.getElement(selectedID) instanceof Edge || tree.getElement(selectedID) instanceof ProcessTree)){
			nameField.setEnabled(true);
			nameField.setText(tree.getElement(selectedID).getName());
		}
		else{
			nameField.setText("");
			nameField.setEnabled(false);
		}
	}
	
	private void updateEdgeTextField(){
		edgeTextField.setEnabled(true);
		edgeTextField.setText("");
		UUID selectedID = graphMouseListener.getSelectedID();
		UUID previouslySelectedID = graphMouseListener.getPreviouslySelectedID();
		if(selectedID == null || previouslySelectedID == null){
			edgeTextField.setEnabled(false);
			return;
		}
		if(tree.getElement(previouslySelectedID) instanceof Block && tree.getElement(selectedID) instanceof Node){
			String from = tree.getElement(previouslySelectedID).getName();
			String to = tree.getElement(selectedID).getName();
			if(from.equalsIgnoreCase("")){
				from = tree.getType((Node)tree.getElement(previouslySelectedID)).toString();
			}
			if(to.equalsIgnoreCase("")){
				to = tree.getType((Node)tree.getElement(selectedID)).toString();
			}
			
			edgeTextField.setText("From: " + from + " -> To: " + to);
		}
		else{
			edgeTextField.setEnabled(false);
		}
	}
	
	private void updateTypeComboBox(){
		typeCombo.removeAllItems();
		UUID selectedID = graphMouseListener.getSelectedID();
		if(!(tree.getElement(selectedID) instanceof Node)){
			return;
		}
		Node selectedNode = tree.getNode(selectedID);
		// so we have a node, now we have to define when we can change what to what
		// in principle we only have to check the number of outgoing edges:
		//	0: everything
		//	1: no auto/manual
		// 	2+: only blocks (no event/auto/manual)
		List<Edge> outgoingEdges = new ArrayList<>();
		List<String> types = new ArrayList<>();
		if(selectedNode instanceof Block){
			outgoingEdges = ((Block)selectedNode).getOutgoingEdges();
		}		
		for(Type t: ProcessTree.Type.values()){
			if(t.equals(ProcessTree.Type.MANTASK) || t.equals(ProcessTree.Type.AUTOTASK)){
				if(outgoingEdges.size() == 0){
					types.add(t.name());
				}
			}
			else if(t.equals(ProcessTree.Type.MESSAGE) || t.equals(ProcessTree.Type.TIMEOUT)){
				if(outgoingEdges.size() <= 1){
					types.add(t.name());
				}
			}
			else {
				types.add(t.name());
			}
		}
		Collections.sort(types);
		for(String t: types){
			typeCombo.addItem(ProcessTree.Type.valueOf(t));
		}
		
	}
	
	private void createChildSlider(JPanel panel, int row, int column, int width, int height){
		JLabel label = new JLabel("Next child's position:");
		panel.add(label, "" + column + ", " + row + ", " + (column + 3) + ", " + row + ", l, c");
		
		childIndex.setName("Index of Child");
		childIndex.setEnabled(false);
		childIndex.setPreferredSize(new Dimension(4 * width, height));
		panel.add(childIndex, "" + (column + 4) + ", " + row + ", " + (column + 7) + ", " + row + ", l, c");
		components.add(childIndex);
	}
	
	private void createLeftRightButton(JPanel panel, int row, int column, int width, int height){
		JButton leftButton = new JButton("<");
		leftButton.addActionListener(new EditorActionListenerMoveNode(tree, graphMouseListener, graphPanel, true));
		leftButton.setPreferredSize(new Dimension(width, height));
		panel.add(leftButton, "" + column + ", " + row + ", " + column + ", " + row + ", l, c");
		components.add(leftButton);
		localComponents.add(leftButton);
		
		JLabel label = new JLabel("Move children left/right");
		panel.add(label, "" + (column + 2) + ", " + row + ", " + (column + 5) + ", " + row + ", c, c");
		JButton rightButton = new JButton(">");
		rightButton.addActionListener(new EditorActionListenerMoveNode(tree, graphMouseListener, graphPanel, false));
		rightButton.setPreferredSize(new Dimension(width, height));
		panel.add(rightButton, "" + (column + 7) + ", " + row + ", " + (column + 7) + ", " + row + ", l, c");
		components.add(rightButton);
		localComponents.add(rightButton);
	}
	
	private void createAddButton(JPanel panel, Type type, int row, int column, int width, int height){
		JButton addNode = new JButton(renderOperator(type));
		addNode.addActionListener(new EditorActionListenerAddNode(tree, graphMouseListener, graphPanel, type, nameField, childIndex));
		addNode.setPreferredSize(new Dimension(width, height));
		panel.add(addNode, "" + column + ", " + row + ", " + column + ", " + row + ", l, c");
		components.add(addNode);
		localComponents.add(addNode);
	}
	
	
	
	private void createBlockingHidingToggleButtons(JPanel panel, int row, int column, int width, int height){
		JButton toggleBlocking = new JButton("Toggle Blocking");
		toggleBlocking.setPreferredSize(new Dimension(3 * width, height));
		toggleBlocking.addActionListener(new EditorActionListenerToggleBlockingHiding(tree, graphMouseListener, graphPanel, true, false));
		panel.add(toggleBlocking, "" + (column + 1) + ", " + row + ", " + (column + 3) + ", " + row + ", l, c");
		components.add(toggleBlocking);
		localComponents.add(toggleBlocking);
		
		JButton toggleHiding = new JButton("Toggle Hiding");
		toggleHiding.setPreferredSize(new Dimension(3 * width, height));
		toggleHiding.addActionListener(new EditorActionListenerToggleBlockingHiding(tree, graphMouseListener, graphPanel, false, true));
		panel.add(toggleHiding, "" + (column + 4) + ", " + row + ", " + (column + 6) + ", " + row + ", l, c");
		components.add(toggleHiding);
		localComponents.add(toggleHiding);
	}
	
	private void createAddEdgeButton(JPanel panel, int row, int column, int width, int height){
		edgeTextField = new JTextField();
		edgeTextField.setEditable(false);
		edgeTextField.setPreferredSize(new Dimension(6 * width, height));
		panel.add(edgeTextField, "" + column + ", " + row + ", " + (column + 5) + ", " + row + ", l, c");
		JButton addEdgeButton = new JButton("Add Edge");
		addEdgeButton.setPreferredSize(new Dimension(2 * width, height));
		addEdgeButton.addActionListener(new EditorActionListenerAddEdge(tree, graphMouseListener, graphPanel, childIndex));
		panel.add(addEdgeButton, "" + (column + 6) + ", " + row + ", " + (column + 7) + ", " + row + ", l, c");
		components.add(addEdgeButton);
		localComponents.add(addEdgeButton);
	}
	
	private void createOperatorChanger(JPanel panel, int row, int column, int width, int height){
		typeCombo = new JComboBox<>();
		typeCombo.setPreferredSize(new Dimension(3 * width, height));
		panel.add(typeCombo, "" + (column + 2) + ", " + row + ", " + (column + 4) + ", " + row + ", l, c");
		
		JButton changeType = new JButton("Change Type");
		changeType.setPreferredSize(new Dimension(3 * width, height));
		changeType.addActionListener(new EditorActionListenerChangeType(tree, graphMouseListener, graphPanel, typeCombo));
		panel.add(changeType, "" + (column + 5) + ", " + row + ", " + (column + 7) + ", " + row + ", l, c");
		components.add(changeType);
		localComponents.add(changeType);
		
	}
	
	private void createDeleteButton(JPanel panel, int row, int column, int width, int height){
		JButton deleteNode = new JButton("Delete");
		deleteNode.setPreferredSize(new Dimension(4 * width, height));
		deleteNode.addActionListener(new EditorActionListenerDeleteElement(tree, Node.class, Edge.class, graphMouseListener, graphPanel));
		panel.add(deleteNode, "" + column + ", " + row + ", " + (column + 3) + ", " + row + ", l, c");
		components.add(deleteNode);
		localComponents.add(deleteNode);
	}
	
	private ImageIcon renderOperator(Type t){
		ProcessTree localTree = new ProcessTreeImpl();
		Node n = generateNode(t);
		localTree.addNode(n);
		n.setProcessTree(localTree);
		localTree.setRoot(n);
		TreeLayoutBuilder builder = new TreeLayoutBuilder(localTree, null);
		JGraph graph = builder.getJGraph();

		//Required to make the graph do the layout thingy otherwise image is empty
		JFrame frame = new JFrame("");
		JScrollPane scroll = new JScrollPane(graph);
		frame.add(scroll);
		frame.setSize(graph.getSize());
		frame.setPreferredSize(graph.getSize());
		frame.pack();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		Color bg = null; // Use this to make the background transparent
		int inset = 0;
		graph.setOpaque(false); // Use this to make the background transparent
		BufferedImage img = graph.getImage(bg, inset);
		if(img != null){
			return new ImageIcon(img);
		}
		return null;
	}
	
	private void checkEnablement(){
		for(JButton component: localComponents){
			for(ActionListener actListen: component.getActionListeners()){
				if(actListen instanceof EditorActionListener){
					component.setEnabled(((EditorActionListener) actListen).isEnabled());
				}
			}
		}
	}
	
	private Node generateNode(Type t){
		Node n;
		String name = "";
		String message = "";
		switch(t){
			case SEQ:
				n = new AbstractBlock.Seq(name);
				break;
			case AND:
				n = new AbstractBlock.And(name);
				break;
			case XOR:
				n = new AbstractBlock.Xor(name);
				break;
			case DEF:
				n = new AbstractBlock.Def(name);
				break;
			case OR:
				n = new AbstractBlock.Or(name);
				break;
			case LOOPXOR:
				n = new AbstractBlock.XorLoop(name);
				break;
			case LOOPDEF:
				n = new AbstractBlock.DefLoop(name);
				break;
			case PLACEHOLDER:
				n = new AbstractBlock.PlaceHolder(name);
				break;
			case TIMEOUT:
				n = new AbstractEvent.TimeOut(name, message);
				break;
			case MESSAGE:
				n = new AbstractEvent.Message(name, message);
				break;
			case MANTASK:
				n = new AbstractTask.Manual("<html><br>Manual</html>", new HashSet<Originator>());
				break;
			default:
				n = new AbstractTask.Automatic("<html><br>Automatic</html>");
				break;
		}
		return n;
	}
	
}
