package org.processmining.processtree.editor.editorpanel;

import info.clearthought.layout.TableLayout;

import java.awt.Dimension;
import java.util.Collection;
import java.util.List;
import java.util.UUID;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.border.BevelBorder;

import org.processmining.framework.plugin.PluginContext;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.ProcessTreeElement;
import org.processmining.processtree.editor.graphpanel.GraphPanel;
import org.processmining.processtree.editor.interaction.GraphMouseListener;

public class EditorPanelProperties {
	PluginContext context;
	ProcessTree tree;
	int maxWidth;
	int maxHeight;
	GraphMouseListener graphMouseListener;
	GraphPanel graphPanel;
	Collection<JComponent> components;
	JComboBox<Property<?>> addProperty;
	JComboBox<Property<?>> removeProperty;
	private int rows = 1;
	private int heightOfRows = 25;
	
	
	public EditorPanelProperties(PluginContext context, ProcessTree tree, int maxWidth, int maxHeight, GraphMouseListener graphMouseListener, GraphPanel graphPanel, Collection<JComponent> components){
		this.context = context;
		this.tree = tree;
		this.maxWidth = maxWidth;
		this.maxHeight = maxHeight;
		this.graphMouseListener = graphMouseListener;
		this.graphPanel = graphPanel;
		this.components = components;
	}
	
	public int getHeight(){
		// return the height in terms of elements
		// added some for the border
		return rows * heightOfRows + 24;
	}
	
	public JComponent createProperties(){
		int columns = 4;
		// scaling for the border
		int width = (maxWidth - 8) / columns;
		double[][] layout = new double[2][Math.max(columns, rows)];
		for(int i = 0; i < columns; i++){
			layout[0][i] = width;
		}
		for(int i = 0; i < rows; i++){
			layout[1][i] = heightOfRows;
		}		
		JPanel panel = new JPanel();
		panel.setLayout(new TableLayout(layout));
		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder("Properties"), BorderFactory.createBevelBorder(BevelBorder.LOWERED)));
		
		
		int offset = 0;
		createSelectionRemovalProperties(panel, offset, 0, width, heightOfRows);
		
		
		return panel;
	}
	
	private void createSelectionRemovalProperties(JPanel panel, int row, int column, int width, int height){
		addProperty = new JComboBox<Property<?>>();
		addProperty.setPreferredSize(new Dimension(width, height));
		removeProperty = new JComboBox<Property<?>>();
		removeProperty.setPreferredSize(new Dimension(width, height));
		JButton addPropertyButton = new JButton("Add");
		addPropertyButton.setPreferredSize(new Dimension(width, height));
		JButton removePropertyButton = new JButton("Remove");
		removePropertyButton.setPreferredSize(new Dimension(width, height));
		addPropertyButton.addActionListener(new EditorActionListenerPropertyButton(tree, addProperty, true, graphMouseListener));
		removePropertyButton.addActionListener(new EditorActionListenerPropertyButton(tree, removeProperty, false, graphMouseListener));
		// now put them on the panel
		panel.add(addProperty, "" + column + ", " + row + ", " + column + ", " + row + ", l, c");
		panel.add(addPropertyButton, "" + (column + 1) + ", " + row + ", " + (column + 1) + ", " + row + ", l, c");
		panel.add(removeProperty, "" + (column + 2) + ", " + row + ", " + (column + 2) + ", " + row + ", l, c");
		panel.add(removePropertyButton, "" + (column + 3) + ", " + row + ", " + (column + 3) + ", " + row + ", l, c");
		components.add(addProperty);
		components.add(removeProperty);
		components.add(addPropertyButton);
		components.add(removePropertyButton);
	}
	
	public void update(){
		updatePropertyCombos();
	}
	
	private void updatePropertyCombos(){
		UUID selectedID = graphMouseListener.getSelectedID();
		if(selectedID == null){
			return;
		}
		ProcessTreeElement element = tree.getElement(selectedID);
		if(element == null){
			return;
		}
		Property<?> selectedAddProperty = (Property<?>) addProperty.getSelectedItem();
		addProperty.removeAllItems();
		Property<?> selectedRemoveProperty = (Property<?>) removeProperty.getSelectedItem();
		removeProperty.removeAllItems();
		List<Property<?>> possibleProperties = EditorPanelAuxiliary.getAllowedProperties(context, element);
		for(Property<?> possibleProp: possibleProperties){
			addProperty.addItem(possibleProp);
		}		
		// the removeProperties combo is simply the properties currently associated to the element
		for(Property<?> prop: element.getIndependentProperties().keySet()){
			removeProperty.addItem(prop);
		}
		addProperty.setSelectedItem(selectedAddProperty);
		removeProperty.setSelectedItem(selectedRemoveProperty);
	}
}
