package org.processmining.processtree.editor.interaction;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Set;
import java.util.UUID;

import org.jgraph.JGraph;
import org.jgraph.graph.DefaultGraphCell;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.editor.editorpanel.EditorPanel;
import org.processmining.processtree.editor.graphpanel.GraphPanel;
import org.processmining.processtree.editor.propertypanel.PropertyPanel;
import org.processmining.processtree.visualization.tree.TreeLayoutBuilder;

public class GraphMouseListener extends MouseAdapter {
	ProcessTree tree;
	UUID selectedID;
	UUID previouslySelectedID;
	JGraph graph;
	GraphPanel graphPanel;
	PropertyPanel propertyPanel;
	EditorPanel editorPanel;
	Set<UUID> foldedNodes;
	
	public GraphMouseListener(ProcessTree tree, JGraph graph, Set<UUID> foldedNodes){
		this.tree = tree;
		this.graph = graph;
		this.foldedNodes = foldedNodes;
	}
	
	public void addGraphPanel(GraphPanel graphPanel){
		this.graphPanel = graphPanel;
	}
	
	public void addPropertyPanel(PropertyPanel propertyPanel){
		this.propertyPanel = propertyPanel;
	}
	
	public void addEditorPanel(EditorPanel editorPanel){
		this.editorPanel = editorPanel;
	}
	
	public void updateGraph(JGraph graph){
		this.graph = graph;
	}
	
	public UUID getSelectedID(){
		return selectedID;
	}
	
	public UUID getPreviouslySelectedID(){
		return previouslySelectedID;
	}
	
	/**
	 * Sets the selected ID, can be used for variables and originators (it will raise a mousePressed event)
	 * @param newID
	 */
	public void setSelectedID(UUID newID){
		this.previouslySelectedID = this.selectedID;
		this.selectedID = newID;
		mousePressed(null);
	}
	
	public void mousePressed(MouseEvent e){
		// get the cell
		if(e != null){
			int x = e.getX();
			int y = e.getY();
			DefaultGraphCell cell = (DefaultGraphCell) graph.getFirstCellForLocation(x, y);
			previouslySelectedID = selectedID;
			if(cell != null){
				selectedID = (UUID) cell.getAttributes().get(TreeLayoutBuilder.ID);
			}
			else{
				selectedID = tree.getID();
			}
			// now do something with double clicks
			if(e.getButton() != MouseEvent.BUTTON1 && selectedID != null && previouslySelectedID != null && selectedID.equals(previouslySelectedID)){
				// we have a double click
				// check if it is a node
				if(tree.getElement(selectedID) instanceof Node){
					if(foldedNodes.contains(selectedID)){
						foldedNodes.remove(selectedID);
					}
					else{
						foldedNodes.add(selectedID);
					}
					graphPanel.redraw();
				}
			}
		}
		// notify the property panel and the editorpanel
		propertyPanel.mousePressed(selectedID);
		editorPanel.mousePressed();
	}
}
