package org.processmining.processtree.impl;

import java.util.UUID;

import org.processmining.processtree.Edge;
import org.processmining.processtree.Event;

public abstract class AbstractEvent extends AbstractBlock implements Event {

	private String message;

	public AbstractEvent(String name, String message) {
		super(name);
		this.message = message;
	}
	
	public AbstractEvent(UUID id, String name, String message) {
		super(id, name);
		this.message = message;
	}

	public AbstractEvent(String name, String message, Edge incoming,
			Edge outgoing) {
		super(name);
		this.addIncomingEdge(incoming);
		this.addOutgoingEdge(outgoing);
		this.message = message;
	}
	
	public AbstractEvent(UUID id, String name, String message, Edge incoming,
			Edge outgoing) {
		super(id, name);
		this.addIncomingEdge(incoming);
		this.addOutgoingEdge(outgoing);
		this.message = message;
	}
	
	public AbstractEvent(Event e){
		super(e);
		this.message = e.getMessage();
	}
	
	@Override
	public boolean isChangeable(){
		return false;
	}
	
	@Override
	public void setChangeable(boolean changeable){
		// just do nothing
	}

	/**
	 * Returns the message associated to this event
	 * 
	 * @return
	 */
	public String getMessage() {
		return message;
	}
	
	public void setMessage(String message){
		this.message = message;
	}

	public static class TimeOut extends AbstractEvent implements Event.TimeOut {

		public TimeOut(String name, String message, Edge incoming, Edge outgoing) {
			super(name, message, incoming, outgoing);
		}
		
		public TimeOut(UUID id, String name, String message, Edge incoming, Edge outgoing) {
			super(id, name, message, incoming, outgoing);
		}

		public TimeOut(String name, String message) {
			super(name, message);
		}
		
		public TimeOut(UUID id, String name, String message) {
			super(id, name, message);
		}
		
		public TimeOut(Event.TimeOut t){
			super(t);
		}

		@Override
		public boolean orderingOfChildernMatters() {
			return false;
		}

		@Override
		public boolean expressionsOfOutgoingEdgesMatter() {
			return false;
		}

		@Override
		public String toStringShort() {
			return "Timeout";
		}
	}

	public static class Message extends AbstractEvent implements Event.Message {

		public Message(String name, String message, Edge incoming, Edge outgoing) {
			super(name, message, incoming, outgoing);
		}
		
		public Message(UUID id, String name, String message, Edge incoming, Edge outgoing) {
			super(id, name, message, incoming, outgoing);
		}

		public Message(String name, String message) {
			super(name, message);
		}
		
		public Message(UUID id, String name, String message) {
			super(id, name, message);
		}
		
		public Message(Event.Message m){
			super(m);
		}

		@Override
		public boolean orderingOfChildernMatters() {
			return false;
		}

		@Override
		public boolean expressionsOfOutgoingEdgesMatter() {
			return false;
		}

		@Override
		public String toStringShort() {
			return "Message";
		}
	}

}
