package org.processmining.processtree.ptml;

import java.util.UUID;

import org.processmining.processtree.Block;
import org.processmining.processtree.ProcessTree;
import org.xmlpull.v1.XmlPullParser;


public abstract class PtmlBlock extends PtmlNode {
	boolean changeable;
	
	public PtmlBlock(String tag, UUID id) {
		super(tag, id);
		changeable = false;
	}
	
	protected void importAttributes(XmlPullParser xpp, Ptml ptml) {
		super.importAttributes(xpp, ptml);
		importDowngradeable(xpp, ptml);
	}
	
	private void importDowngradeable(XmlPullParser xpp, Ptml ptml) {
		String value = xpp.getAttributeValue(null, "changeable");
		if (value != null) {
			changeable = new Boolean(value);
		}
		ptml.log(value != null ? value : "<no value>");	
	}

	protected String exportAttributes(Ptml ptml) {
		return super.exportAttributes(ptml) + exportChangeable(ptml);
	}

	private String exportChangeable(Ptml ptml) {
		if (changeable) {
			return exportAttribute("changeable", "true", ptml);
		}
		return "";
	}
	
	public void marshall(ProcessTree tree, Block element) {
		super.marshall(tree, element);
		changeable = element.isChangeable();
	}
}
