package org.processmining.processtree.ptml;

import java.util.Map;
import java.util.UUID;

import org.processmining.processtree.Expression;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.impl.ExpressionImpl;
import org.xmlpull.v1.XmlPullParser;

public class PtmlExpression extends PtmlProcessTreeElement {
	public final static String TAG = "expression";
	
	private String expression;

	public PtmlExpression(UUID id) {
		super(TAG, id);
	}

	public void unmarshall(ProcessTree tree, Map<UUID, Expression> map) {
		Expression expr = new ExpressionImpl(id, name, expression);
		tree.addExpression(expr);
		unmarshall(tree, map, expr);
	}
	
	public PtmlExpression(String tag, UUID id) {
		super(tag, id);
	}
	
	public void unmarshall(ProcessTree tree, Map<UUID, Expression> map, Expression expr) {
		map.put(id, expr);
		if(propertyIndependent != null){
			propertyIndependent.unmarshall(expr);
		}
		if(propertyDependent != null){
			propertyDependent.unmarshall(expr);
		}
	}
	
	protected void importAttributes(XmlPullParser xpp, Ptml ptml) {
		super.importAttributes(xpp, ptml);
		importExpression(xpp, ptml);
	}
	
	private void importExpression(XmlPullParser xpp, Ptml ptml) {
		String value = xpp.getAttributeValue(null, "expression");
		if (value != null) {
			expression = value;
		}
		ptml.log(expression != null ? value : "<no expression>");
	}
	
	protected String exportAttributes(Ptml ptml) {
		return super.exportAttributes(ptml) + exportExpression(ptml);
	}

	private String exportExpression(Ptml ptml) {
		return exportAttribute("expression", expression, ptml);
	}

	public void marshall(ProcessTree tree, Map<Expression, UUID> expressionMap, Expression expr) {
		marshall(tree, expr);
		expression = expr.getExpression();
		expressionMap.put(expr, id);
		
	}
}
