package org.processmining.processtree.ptml;

import java.util.UUID;

import org.processmining.framework.plugin.PluginContext;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.ProcessTreeElement;
import org.xmlpull.v1.XmlPullParser;

/**
 * New element added, for most import and export functionality
 * @author DSchunse
 *
 */
public class PtmlProcessTreeElement extends PtmlId {
	protected String name;
	protected PtmlPropertyIndependent propertyIndependent;
	protected PtmlPropertyDependent propertyDependent;
	
	public PtmlProcessTreeElement(String tag, UUID id) {
		super(tag, id);
		name = null;
		propertyIndependent = null;
		propertyDependent = null;
	}
	
	protected void importAttributes(XmlPullParser xpp, Ptml ptml) {
		super.importAttributes(xpp, ptml);
		importName(xpp, ptml);
	}

	private void importName(XmlPullParser xpp, Ptml ptml) {
		String value = xpp.getAttributeValue(null, "name");
		if (value != null) {
			name = value;
		}
		ptml.log(name != null ? name : "<no name>");
	}

	protected String exportAttributes(Ptml ptml) {
		return super.exportAttributes(ptml) + exportName(ptml);
	}

	private String exportName(Ptml ptml) {
		if (name != null) {
			return exportAttribute("name", name, ptml);
		}
		return "";
	}
	
	protected String exportElements(Ptml ptml){
		String s = super.exportElements(ptml);
		// property export
		if(propertyIndependent != null){
			s += propertyIndependent.exportElement(ptml);
		}
		if(propertyDependent != null){
			s += propertyDependent.exportElement(ptml);
		}
		return s;
	}
	
	protected boolean importElements(PluginContext context, XmlPullParser xpp, Ptml ptml) {
		PtmlPropertyIndependent propertyInd = null;
		if(xpp.getName().equals(PtmlPropertyIndependent.TAG)){
			propertyInd = new PtmlPropertyIndependent();
		}
		if(propertyInd != null){
			propertyIndependent = propertyInd;
			propertyIndependent.importElement(context, xpp, ptml);
			return true;
		}
		
		PtmlPropertyDependent propertyDep = null;
		if(xpp.getName().equals(PtmlPropertyDependent.TAG)){
			propertyDep = new PtmlPropertyDependent();
		}
		if(propertyDep != null){
			propertyDependent = propertyDep;
			propertyDependent.importElement(context, xpp, ptml);
			return true;
		}
		return false;
	}
	
	public void marshall(ProcessTree tree, ProcessTreeElement element) {
		id = element.getID();
		name = element.getName();
		if(!element.getIndependentProperties().isEmpty()){
			propertyIndependent = new PtmlPropertyIndependent();
			propertyIndependent.marshall(element.getIndependentProperties());
		}
		if(!element.getDependentProperties().isEmpty()){
			propertyDependent = new PtmlPropertyDependent();
			propertyDependent.marshall(element.getDependentProperties());
		}
	}

}
