package org.processmining.processtree.ptml;

import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.processmodel.Property;
import org.xmlpull.v1.XmlPullParser;


public class PtmlProperty extends PtmlElement {

	/**
	 * PTML tag.
	 */
	public final static String TAG = "property";
	private Property<?> property;
	private Object value;
	private String valueEntries;
	
	
	public PtmlProperty() {
		super(TAG);
	}
	
	public PtmlProperty(Property<?> prop) {
		super(TAG);
		property = prop;
		valueEntries = "";
	}
		
	public void marshall(Property<?> prop, Object val) {
		property = prop;
		value = val;
	}
	
	protected String exportAttributes(Ptml ptml) {
		return ptml.exportAttribute("name", property.getName(), ptml) + ptml.exportAttribute("id", property.getID().toString(), ptml);
	}
	
	protected String exportElements(Ptml ptml) {
		String s = super.exportElements(ptml);
		s += property.marshall(value);
		return s;
	}
	
	protected boolean importElements(PluginContext context, XmlPullParser xpp, Ptml ptml){
		// we always have entries, so lets not bother with it here :-)
		// lets parse any sub thing we have here :-D
		valueEntries += xpp.getText();
		String enclosingName = xpp.getName();
		boolean wasClosing = true;
		try {
			boolean done = xpp.getText().endsWith("/>");
			wasClosing = done;
			while(!done){
				int name = xpp.next();
				String tag = xpp.getName();
				if(name == XmlPullParser.END_TAG && tag.equalsIgnoreCase(enclosingName)){
					done = true;
				}
				if(name == XmlPullParser.START_TAG){
					valueEntries += xpp.getText();
				}
				if(name == XmlPullParser.END_TAG && !xpp.getText().endsWith("/>")){
					// we have a normal closing tag, else we would duplicate the text
					valueEntries += xpp.getText();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return !wasClosing;
	}
	
	@SuppressWarnings("unchecked")
	protected Pair<Class<? extends Property<?>>, Object> unmarshall(){
		return new Pair<Class<? extends Property<?>>, Object>((Class<? extends Property<?>>) property.getClass(), property.unmarshall(valueEntries));
	}

}
