package org.processmining.processtree.ptml;

import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.Pair;
import org.processmining.plugins.properties.annotations.PropertableElementProperty;
import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.processtree.ProcessTreeElement;
import org.xmlpull.v1.XmlPullParser;

public class PtmlPropertyDependent extends PtmlElement {
	/**
	 * PTML tag.
	 */
	public final static String TAG = "propertyDependent";
	private List<PtmlProperty> propertyList;
	
	public PtmlPropertyDependent() {
		super(TAG);
		propertyList = new ArrayList<PtmlProperty>();	
	}
	
	protected String exportElements(Ptml ptml) {
		String s = super.exportElements(ptml);
		for (PtmlProperty property : propertyList) {
			s += property.exportElement(ptml);
		}
		return s;
	}
	
	public void marshall(AbstractMap<Property<?>, Object> abstractMap) {
		for(Property<?> prop: abstractMap.keySet()){
			PtmlProperty ptmlProp = new PtmlProperty();
			ptmlProp.marshall(prop, abstractMap.get(prop));
			propertyList.add(ptmlProp);
		}
	}
	
	@SuppressWarnings("unchecked")
	protected boolean importElements(PluginContext context, XmlPullParser xpp, Ptml ptml){
		PtmlProperty prop = null;
		if(xpp.getName().equals(PtmlProperty.TAG)){
			Set<Class<?>> properties = context.getPluginManager().getKnownClassesAnnotatedWith(PropertableElementProperty.class);
			Long id = Long.valueOf(xpp.getAttributeValue(null, "id"));
			for(Class<?> propClass: properties){
				try {
					if(((Class<? extends Property<?>>)propClass).newInstance().getID().equals(id)){
						prop = new PtmlProperty(((Class<? extends Property<?>>) propClass).newInstance());
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		if(prop != null){
			propertyList.add(prop);
			prop.importElement(context, xpp, ptml);
			return true;
		}
		
		return false;
	}
	
	public void unmarshall(ProcessTreeElement element) {
		for(PtmlProperty prop: propertyList){
			Pair<Class<? extends Property<?>>, Object> keyValue = prop.unmarshall();
			try {
				element.setDependentProperty(keyValue.getFirst(), keyValue.getSecond());
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
}
