package org.processmining.processtree.visualization.extrainforenderer.colourcoding.impl;

import java.awt.Color;

import org.processmining.plugins.properties.processmodel.Property;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.visualization.extrainforenderer.colourcoding.ColourCoding;

public class ColourCodingImpl implements ColourCoding{
	Property<?> property;
	Double relWidth = 1d;
	boolean independent;
	Double maxValue = null;
	Double meanValue = null;
	//double minValue = 0d;

	public ColourCodingImpl(Property<?> property, Double relWidth, boolean independent) {
		this.property = property;
		if(property == null){
			throw new NullPointerException();
		}
		this.relWidth = relWidth;
		if(relWidth == null){
			throw new NullPointerException();
		}
		this.independent = independent;
	}
	
	public Color giveColour(ProcessTree tree, Node n) throws InstantiationException, IllegalAccessException {
		if(tree == null || n == null){
			throw new NullPointerException();
		}
		
		// lets have a scale between 0 and 255 :-)
		if(maxValue == null){
			maxValue = -Double.MIN_VALUE;
			meanValue = 0d;
			// it has not yet been set
			for(Node np: tree.getNodes()){
				if(independent && np.getIndependentProperty(property) != null){
					maxValue = Math.max(property.getValueDouble(np.getIndependentProperty(property)), maxValue);
					meanValue += property.getValueDouble(np.getIndependentProperty(property));
				}
				else if(!independent && np.getDependentProperty(property) != null){
					maxValue = Math.max(property.getValueDouble(np.getDependentProperty(property)), maxValue);
					meanValue += property.getValueDouble(np.getDependentProperty(property));
				}
			}
		}
		meanValue = meanValue / tree.getNodes().size();
		double mean = 0;
		boolean noNull = false;
		if(independent && n.getIndependentProperty(property) != null){
			mean = property.getValueDouble(n.getIndependentProperty(property));
			noNull = true;
		}
		if(!independent && n.getDependentProperty(property) != null){
			mean = property.getValueDouble(n.getDependentProperty(property));
			noNull = true;
		}
		// lets scale this mean
		mean = mean * (maxValue == null ? 0 : (255 / maxValue));
		// now lets make an int out of it
		if(noNull){
			return new Color((int)mean, 0, 0);
		}
		else{
			return new Color(255, 255, 255);
		}
	}

	public double giveRelWidth() {
		return this.relWidth;
	}

	public Property<?> coloursProperty() {
		return property;
	}

	public boolean isIndependent() {
		return independent;
	}

}
