package org.processmining.processtree.visualization.jgraph;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.geom.AffineTransform;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.util.StringTokenizer;

import org.jgraph.graph.CellView;
import org.jgraph.graph.DefaultGraphCell;

import com.jgraph.components.labels.MultiLineVertexRenderer;

public class BPMNOperatorRenderer extends MultiLineVertexRenderer {

	public static enum OperatorType {
		XOR("X"), OR("O"), AND("+"), DEF("D"), START(""), END(""), PLACEHOLDER("");
		private final String s;

		OperatorType(String s) {
			this.s = s;
		}

		public String toString() {
			return s;
		}
	};

	/**
	 * 
	 */
	private static final long serialVersionUID = 4910262989128580150L;
	protected OperatorType type;
	private float[] pattern = new float[] { 5, 5 };

	@Override
	protected void paintShapeBorder(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;
		Stroke s = g2d.getStroke();
		if (type == OperatorType.PLACEHOLDER) {
			g2d.setStroke(new BasicStroke(borderWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 10.0F, pattern,
					0F));
		}
		Dimension d = getSize();
		int b = borderWidth;
		if (shape == SHAPE_CIRCLE) {
			g.drawOval(b - 1, b - 1, d.width - b - 1, d.height - b - 1);
		} else {
			super.paintShapeBorder(g);
		}
		g2d.setStroke(s);
	}

	@Override
	protected void paintComponent(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;

		GeneralPath gatewayDecorator = new GeneralPath();
		double s = (getWidth() - 2.0 * borderWidth) / 33.;
		switch (type) {
			case AND :
				drawParallel(g2d, gatewayDecorator, s);
				break;
			case END :
				break;
			case OR :
				drawInclusive(g2d, gatewayDecorator, s);
				break;
			case DEF :
				drawEventbased(g2d, gatewayDecorator, s);
				break;
			case START :
				break;
			case XOR :
				drawExclusive(g2d, gatewayDecorator, s);
				break;
			case PLACEHOLDER :
				break;

		}
		//super.paintComponent(g);

		AffineTransform at = new AffineTransform();
		//at.scale(s, s);
		//gatewayDecorator.transform(at);
		//at = new AffineTransform();
		at.translate(borderWidth, borderWidth);
		gatewayDecorator.transform(at);

		g.setColor(Color.BLACK);
		if (type == OperatorType.XOR || type == OperatorType.AND) {
			g2d.fill(gatewayDecorator);
		} else {
			g2d.draw(gatewayDecorator);
		}
	}

	public void installAttributes(CellView view) {
		super.installAttributes(view);
		type = (OperatorType) ((DefaultGraphCell) view.getCell()).getUserObject();
	}

	private void drawExclusive(Graphics2D g2d, GeneralPath gatewayDecorator, double s) {

		parseSVG(g2d, gatewayDecorator, "M 11.5 9.5 L 13.5 9.5 L 20.5 22.5 L 18.5 22.5 L 11.5 9.5", s);
		parseSVG(g2d, gatewayDecorator, "M 11.5 22.5 L 18.5 9.5 L 20.5 9.5 L 13.5 22.5 L 11.5 22.5", s);
		g2d.setStroke(new BasicStroke(1F));

	}

	public static void parseSVG(Graphics2D g2d, GeneralPath gatewayDecorator, String path, double s) {
		StringTokenizer tok = new StringTokenizer(path, " ,");
		while (tok.hasMoreTokens()) {
			String op = tok.nextToken();
			double x = Double.parseDouble(tok.nextToken());
			double y = Double.parseDouble(tok.nextToken());

			if (op.equals("M")) {
				gatewayDecorator.moveTo(s * x, s * y);
			} else {
				gatewayDecorator.lineTo(s * x, s * y);
			}
		}
	}

	private void drawEventbased(Graphics2D g2d, GeneralPath gatewayDecorator, double s) {

		gatewayDecorator.append(new Ellipse2D.Double(s * 5.5, s * 5.5, s * 21, s * 21), false);
		gatewayDecorator.append(new Ellipse2D.Double(s * 7.5, s * 7.5, s * 17, s * 17), false);

		parseSVG(g2d, gatewayDecorator, "M 16 10 L 21.7 14.15 L 19.5 20.85 L 12.5 20.85 L 10.3 14.15", s);
		gatewayDecorator.closePath();
		g2d.setStroke(new BasicStroke((float) s * 1F));
	}

	private void drawInclusive(Graphics2D g2d, GeneralPath gatewayDecorator, double s) {
		gatewayDecorator.append(new Ellipse2D.Double(s * 8.5, s * 8.5, s * 15, s * 15), false);
		g2d.setStroke(new BasicStroke((float) (s * 2.5)));
	}

	@SuppressWarnings("unused")
	private void drawComplex(Graphics2D g2d, GeneralPath gatewayDecorator, double s) {

		parseSVG(g2d, gatewayDecorator,
				"M 8.5,16 L 23.5,16 M 16,8.5 L 16,23.5	M 10.5,10.5 L 21.5,21.5	M 10.5,21.5 L 21.5,10.5", s);
		g2d.setStroke(new BasicStroke((float) s * 3F));
	}

	private void drawParallel(Graphics2D g2d, GeneralPath gatewayDecorator, double s) {
		parseSVG(
				g2d,
				gatewayDecorator,
				"M 8.5 15 L 8.5 17 L 15 17 L 15 23.5 L 17 23.5 L 17 17 L 23.5 17 L 23.5 15 L 17 15 L 17 8.5 L 15 8.5 L 15 15 L 8.5 15",
				s);
	}

	/*-
	<circle
		id="frame" cx="16" cy="16" r="15"
		stroke="black" fill="white" stroke-width="1"
	/>	 
	 */
	// startEvent

	/*-
	 <circle
		id="frame" cx="16" cy="16" r="15"
		stroke="black" fill="white" stroke-width="1"
	/>
	<circle
		id="inner" cx="16" cy="16" r="12"
		stroke="black" fill="none" stroke-width="1"
	/>
	 */

}
