package org.processmining.processtree.visualization.jgraph;

import java.awt.Shape;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Path2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import org.jgraph.graph.CellView;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.EdgeRenderer;
import org.processmining.processtree.visualization.bpmn.TreeBPMNLayoutBuilder;
import org.processmining.processtree.visualization.tree.TreeLayoutBuilder;

public class ConfigurableEdgeRenderer extends EdgeRenderer {

	private static final long serialVersionUID = -4562423326343185424L;
	protected Boolean hideable;
	protected Boolean blockable;

	/**
	 * Paint the current view's direction. Sets tmpPoint as a side-effect such
	 * that the invoking method can use it to determine the connection point to
	 * this decoration.
	 */
	@Override
	protected Shape createLineEnd(int size, int style, Point2D src, Point2D dst) {
		if (src == null || dst == null)
			return null;

		Shape basis = super.createLineEnd(size, style, src, dst);
		boolean horizontal = false;
		if (Math.abs(src.getX() - dst.getX()) > Math.abs(src.getY() - dst.getY())) {
			horizontal = true;
		}

		if (blockable && hideable) {
			double x1, x2;
			double y1, y2;

			double h;
			double w;
			if (horizontal) {
				h = TreeBPMNLayoutBuilder.CONFIGSIZE + 2.33 / 3.0 * TreeBPMNLayoutBuilder.CONFIGSIZE;
				w = TreeBPMNLayoutBuilder.CONFIGSIZE;
			} else {
				h = (blockable ? TreeBPMNLayoutBuilder.CONFIGSIZE : 2.33 / 3.0 * TreeBPMNLayoutBuilder.CONFIGSIZE);
				w = TreeBPMNLayoutBuilder.CONFIGSIZE;
			}

			if (src.getX() < dst.getX() && horizontal) {
				x1 = dst.getX() - 2 - w;
				x2 = x1;
				y1 = dst.getY() - h;
				y2 = y1 + TreeLayoutBuilder.CONFIGSIZE;
			} else if (horizontal) {
				x1 = dst.getX() + 2 + size;
				x2 = x1;
				y1 = dst.getY() - h;
				y2 = y1 + TreeLayoutBuilder.CONFIGSIZE;
			} else if (src.getY() < dst.getY()) {
				x1 = dst.getX() - w;
				x2 = x1 + TreeLayoutBuilder.CONFIGSIZE;
				y1 = dst.getY() - 2 - h;
				y2 = y1;
			} else {
				x1 = dst.getX() - w;
				x2 = x1 + TreeLayoutBuilder.CONFIGSIZE;
				y1 = dst.getY() + 2 + size;
				y2 = y1;
			}
			Path2D path = new Path2D.Float();
			path.moveTo(x1, y1);
			addBlockable(path, x1, y1, size);
			path.moveTo(x2, y2);
			addHideable(path, x2, y2, size);
			path.append(basis, false);

			return path;

		} else if (blockable || hideable) {
			double x;
			double y;
			double h = (blockable ? TreeBPMNLayoutBuilder.CONFIGSIZE : 2.33 / 3.0 * TreeBPMNLayoutBuilder.CONFIGSIZE);
			double w = TreeBPMNLayoutBuilder.CONFIGSIZE;
			// determine general direction

			if (src.getX() < dst.getX() && horizontal) {
				x = dst.getX() - 2 - w;
				y = dst.getY() - h;
			} else if (horizontal) {
				x = dst.getX() + 2 + size;
				y = dst.getY() - h;
			} else if (src.getY() < dst.getY()) {
				x = dst.getX() - w;
				y = dst.getY() - 2 - h;
			} else {
				x = dst.getX() - w;
				y = dst.getY() + 2 + size;
			}
			Path2D path = new Path2D.Float();
			path.moveTo(x, y);
			if (blockable) {
				addBlockable(path, x, y, size);
			} else {
				addHideable(path, x, y, size);
			}
			path.append(basis, false);

			return path;
		} else {
			return basis;
		}
	}

	private void addBlockable(Path2D path, double x, double y, int size) {
		Shape circle = new Ellipse2D.Double(x, y, 3 * size, 3 * size);
		Shape rect = new Rectangle2D.Double(x + size / 2, y + size, 2 * size + 1, size);
		Area area = new Area(circle);
		area.subtract(new Area(rect));
		path.append(area, false);
	}

	private void addHideable(Path2D path, double x, double y, int size) {
		path.moveTo(x, y + 1.5 * size);
		path.lineTo(x + .5 * size, y + 2 * size);
		path.curveTo(x + size, y + size, x + 2 * size, y + size, x + 2.33 * size, y + 1.66 * size);
		path.lineTo(x + 2 * size, y + 2 * size);
		path.lineTo(x + 3 * size, y + 2 * size);
		path.lineTo(x + 3 * size, y + size);
		path.lineTo(x + 2.66 * size, y + 1.33 * size);
		path.curveTo(x + 2.33 * size, y + .33 * size, x + .66 * size, y + .33 * size, x, y + 1.5 * size);
	}

	@Override
	public void installAttributes(CellView view) {
		super.installAttributes(view);
		blockable = (Boolean) ((DefaultEdge) view.getCell()).getAttributes().get(TreeBPMNLayoutBuilder.BLOCKABLE);
		hideable = (Boolean) ((DefaultEdge) view.getCell()).getAttributes().get(TreeBPMNLayoutBuilder.HIDEABLE);
	}

}
