package org.processmining.processtree.visualization.jgraph;

import org.jgraph.graph.AbstractCellView;
import org.jgraph.graph.CellView;
import org.jgraph.graph.DefaultCellViewFactory;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.EdgeView;
import org.jgraph.graph.GraphModel;
import org.jgraph.graph.VertexView;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.visualization.Path;
import org.processmining.processtree.visualization.jgraph.BPMNOperatorRenderer.OperatorType;
import org.processmining.processtree.visualization.jgraph.EventRenderer.EventType;

import com.jgraph.components.labels.MultiLineVertexView;

public class TreeCellViewFactory extends DefaultCellViewFactory {

	/**
	 * 
	 */
	private static final long serialVersionUID = -7288009143327540248L;

	public TreeCellViewFactory() {
	}

	@Override
	public CellView createView(GraphModel model, Object cell) {
		CellView view = null;
		if (model.isPort(cell))
			view = createPortView(cell);
		else if (model.isEdge(cell))
			view = createEdgeView(cell);
		else
			view = createTheVertexView(cell);
		return view;
	}

	protected AbstractCellView createTheVertexView(Object cell) {
		Object userObject = ((DefaultGraphCell) cell).getUserObject();
		if (userObject instanceof OperatorType) {
			// operator
			return new BPMNOperatorView(cell);
		} else if (userObject instanceof EventType) {
			return new EventView(cell);
		} else if (userObject instanceof Path) {
			return new VertexView(cell);
		} else if (userObject instanceof Node) {
			return new VertexView(cell);
		} else if (userObject instanceof ProcessTree.Type) {
			return new TreeNodeView(cell);
		}
		return new MultiLineVertexView(cell);
	}

	@Override
	protected EdgeView createEdgeView(Object edge) {
		//EdgeView view = super.createEdgeView(edge);
		return new ConfigurableEdgeView(edge);

	}

}
