package org.processmining.projectedrecallandprecision.framework;

import org.deckfour.xes.classification.XEventClassifier;
import org.processmining.plugins.InductiveMiner.efficienttree.EfficientTreeReduceParameters;
import org.processmining.plugins.InductiveMiner.mining.MiningParameters;

public class CompareParameters implements Cloneable {
	private int k;
	private int maxStatesReachabilityGraph = 2500000;

	private boolean computeRecallFitness = true;
	private boolean computePrecision = true;

	private boolean debug = true;
	private int debugEvery = 10;
	private boolean multiThreading = true;
	private XEventClassifier classifier = MiningParameters.getDefaultClassifier();
	private RecallName recallName;

	private EfficientTreeReduceParameters treeReduceParameters = new EfficientTreeReduceParameters(false);

	public enum RecallName {
		recall, fitness
	}

	public CompareParameters(RecallName recallName) {
		k = 2;
		this.recallName = recallName;
	}
	
	public CompareParameters() {
		k = 2;
		recallName = RecallName.fitness;
	}

	@Override
	public CompareParameters clone() {
		CompareParameters result = new CompareParameters(getRecallName());
		result.k = k;
		result.classifier = classifier;
		result.computePrecision = computePrecision;
		result.computeRecallFitness = computeRecallFitness;
		result.debug = debug;
		result.debugEvery = debugEvery;
		result.maxStatesReachabilityGraph = maxStatesReachabilityGraph;
		result.multiThreading = multiThreading;
		result.treeReduceParameters = result.treeReduceParameters;
		return result;
	}

	/**
	 * 
	 * @param k
	 *            determines whether tuples, triples, etc are used in the
	 *            projection. Notice that > 3 is untested.
	 */
	public CompareParameters(int k) {
		this.k = k;
	}

	public int getK() {
		return k;
	}

	public void setK(int k) {
		this.k = k;
	}

	public int getMaxStatesReachabilityGraph() {
		return maxStatesReachabilityGraph;
	}

	public void setMaxStatesReachabilityGraph(int maxStatesReachabilityGraph) {
		this.maxStatesReachabilityGraph = maxStatesReachabilityGraph;
	}

	public boolean isComputeRecallFitness() {
		return computeRecallFitness;
	}

	public void setComputeRecallFitness(boolean computeRecallFitness) {
		this.computeRecallFitness = computeRecallFitness;
	}

	public boolean isComputePrecision() {
		return computePrecision;
	}

	public void setComputePrecision(boolean computePrecision) {
		this.computePrecision = computePrecision;
	}

	public boolean isDebug() {
		return debug;
	}

	public void setDebug(boolean debug) {
		this.debug = debug;
	}

	public XEventClassifier getClassifier() {
		return classifier;
	}

	public void setClassifier(XEventClassifier classifier) {
		this.classifier = classifier;
	}

	public RecallName getRecallName() {
		return recallName;
	}

	public void setRecallName(RecallName recallName) {
		this.recallName = recallName;
	}

	public boolean isMultiThreading() {
		return multiThreading;
	}

	public void setMultiThreading(boolean multiThreading) {
		this.multiThreading = multiThreading;
	}

	public int getDebugEvery() {
		return debugEvery;
	}

	public void setDebugEvery(int debugEvery) {
		this.debugEvery = debugEvery;
	}

	public EfficientTreeReduceParameters getTreeReduceParameters() {
		return treeReduceParameters;
	}

	public void setTreeReduceParameters(EfficientTreeReduceParameters treeReduceParameters) {
		this.treeReduceParameters = treeReduceParameters;
	}
}
