package org.processmining.projectedrecallandprecision.helperclasses;

import java.util.ArrayDeque;
import java.util.Map;
import java.util.Set;

import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.plugins.InductiveMiner.Pair;

import dk.brics.automaton2.Automaton;
import dk.brics.automaton2.State;
import dk.brics.automaton2.Transition;
import gnu.trove.map.hash.THashMap;
import gnu.trove.procedure.TObjectObjectProcedure;
import gnu.trove.set.hash.THashSet;

public class MapConjunction {

	/**
	 * 
	 * @param conjunction
	 * @param automaton2
	 * @param canceller 
	 * @return map of states in conjunction to multiple states in automaton2
	 */
	public static THashMap<State, Set<State>> mapConjunction(Automaton conjunction, Automaton automaton2, ProMCanceller canceller) {
		ArrayDeque<Pair<State, State>> queue = new ArrayDeque<>();
		queue.add(Pair.of(conjunction.getInitialState(), automaton2.getInitialState()));

		THashMap<State, Set<State>> mapping = new THashMap<>();
		mapping.put(conjunction.getInitialState(), new THashSet<State>());
		mapping.get(conjunction.getInitialState()).add(automaton2.getInitialState());

		Pair<State, State> pair = queue.poll();
		while (pair != null) {
			State conjunctionState = pair.getA();
			State automaton2State = pair.getB();

			//follow each transition
			for (Transition t : conjunctionState.getTransitions()) {
				State nextConjunctionState = t.getDest();
				State nextAutomaton2State = automaton2State.step(t.getMin());

				if (!mapping.containsKey(nextConjunctionState)) {
					mapping.put(nextConjunctionState, new THashSet<State>());
				}
				if (mapping.get(nextConjunctionState).add(nextAutomaton2State)) {
					queue.add(Pair.of(nextConjunctionState, nextAutomaton2State));
				}
				
				if (canceller.isCancelled()) {
					return null;
				}
			}

			if (canceller.isCancelled()) {
				return null;
			}
			
			pair = queue.poll();
		}

		return mapping;
	}
	
	public static Map<State, Set<State>> reverseMapping(THashMap<State, Set<State>> map1, final ProMCanceller canceller) {
		final Map<State, Set<State>> mapReverse = new THashMap<>();
		
		map1.forEachEntry(new TObjectObjectProcedure<State, Set<State>>() {
			public boolean execute(State a, Set<State> b) {
				for (State sb : b) {
					if (!mapReverse.containsKey(sb)) {
						mapReverse.put(sb, new THashSet<State>());
					}
					mapReverse.get(sb).add(a);
				}
				return !canceller.isCancelled();
			}
		});
		
		return mapReverse;
	}

}
