package org.processmining.projectedrecallandprecision.helperclasses;

import java.util.ArrayDeque;

import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.projectedrecallandprecision.helperclasses.automata.DFA;
import org.processmining.projectedrecallandprecision.helperclasses.automata.DFAImplEdgeList.EdgeIterableOutgoing;
import org.processmining.projectedrecallandprecision.helperclasses.automata.operations.StatePair;

import gnu.trove.map.TIntObjectMap;
import gnu.trove.map.hash.TIntObjectHashMap;
import gnu.trove.procedure.TIntObjectProcedure;
import gnu.trove.procedure.TIntProcedure;
import gnu.trove.set.TIntSet;
import gnu.trove.set.hash.TIntHashSet;

public class MapConjunction2 {

	/**
	 * 
	 * @param conjunction
	 * @param automatonA
	 * @param canceller
	 * @return map of states in conjunction to multiple states in automaton2
	 */
	public static TIntObjectMap<TIntSet> mapConjunction(DFA conjunction, DFA automatonA,
			ProMCanceller canceller) {
		ArrayDeque<StatePair> queue = new ArrayDeque<>();
		queue.add(new StatePair(automatonA.getInitialState(), conjunction.getInitialState()));

		TIntObjectMap<TIntSet> mapping = new TIntObjectHashMap<>(10, 0.5f, -1);
		mapping.put(conjunction.getInitialState(), new TIntHashSet(10, 0.5f, -1));
		mapping.get(conjunction.getInitialState()).add(automatonA.getInitialState());

		EdgeIterableOutgoing it = conjunction.getOutgoingEdgesIterator(conjunction.getInitialState());

		StatePair pair = queue.poll();
		while (pair != null) {
			int conjunctionState = pair.stateB;
			int automatonAState = pair.stateA;

			//follow each transition
			for (it.reset(conjunctionState); it.hasNext();) {
				int nextConjunctionState = it.nextTarget();
				int nextAutomatonAState = automatonA.step(automatonAState, it.getActivity());

				if (!mapping.containsKey(nextConjunctionState)) {
					mapping.put(nextConjunctionState, new TIntHashSet(10, 0.5f, -1));
				}
				if (mapping.get(nextConjunctionState).add(nextAutomatonAState)) {
					queue.add(new StatePair(nextAutomatonAState, nextConjunctionState));
				}

				if (canceller.isCancelled()) {
					return null;
				}
			}

			if (canceller.isCancelled()) {
				return null;
			}

			pair = queue.poll();
		}

		return mapping;
	}

	public static TIntObjectMap<TIntSet> reverseMapping(TIntObjectMap<TIntSet> tIntObjectMap,
			final ProMCanceller canceller) {
		final TIntObjectMap<TIntSet> mapReverse = new TIntObjectHashMap<>(10, 0.5f, -1);

		tIntObjectMap.forEachEntry(new TIntObjectProcedure<TIntSet>() {
			public boolean execute(final int a, TIntSet b) {
				b.forEach(new TIntProcedure() {
					public boolean execute(int value) {
						if (!mapReverse.containsKey(value)) {
							mapReverse.put(value, new TIntHashSet(10, 0.5f, -1));
						}
						mapReverse.get(value).add(a);
						return true;
					}
				});

				return !canceller.isCancelled();
			}
		});

		return mapReverse;
	}

}
