package org.processmining.projectedrecallandprecision.helperclasses.automata;

import java.util.Arrays;
import java.util.BitSet;

import org.processmining.framework.plugin.ProMCanceller;

import gnu.trove.list.TIntList;
import gnu.trove.list.TShortList;

/**
 * Provides a fast way to evaluate whether the automaton accepts a trace (after
 * a setup and memory cost).
 * 
 * @author sander
 *
 */
public class RunAutomaton2 {

	final int[][] edges;
	final BitSet accepting;

	public RunAutomaton2(DFA automaton, ProMCanceller canceller) {
		accepting = automaton.getAcceptingStates();

		edges = new int[automaton.getNumberOfStates()][];

		TIntList sources = automaton.getSources();
		TIntList targets = automaton.getTargets();
		TShortList activities = automaton.getActivities();

		/*
		 * We walk backward to make sure that we only have to create each edge
		 * array once (the last activity is the one with the highest number)
		 */
		for (int edge = sources.size() - 1; edge >= 0; edge--) {
			int source = sources.get(edge);
			int target = targets.get(edge);
			short activity = activities.get(edge);
			if (edges[source] == null) {
				edges[source] = new int[activity + 1];
				Arrays.fill(edges[source], -1);
			}
			edges[source][activity] = target;
		}
	}

	public boolean accepts(short[] trace) {
		int state = 0;

		for (int activity : trace) {
			if (edges[state] == null || activity >= edges[state].length) {
				return false;
			}

			state = edges[state][activity];
			if (state == -1) {
				return false;
			}
		}

		return accepting.get(state);
	}
}
