package org.processmining.projectedrecallandprecision.helperclasses.automata.operations;

import java.util.LinkedList;

import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.plugins.InductiveMiner.Pair;
import org.processmining.projectedrecallandprecision.helperclasses.automata.DFA;
import org.processmining.projectedrecallandprecision.helperclasses.automata.DFAImplEdgeList;
import org.processmining.projectedrecallandprecision.helperclasses.automata.DFAImplEdgeList.EdgeIterableOutgoing;

import gnu.trove.map.TObjectIntMap;
import gnu.trove.map.hash.TObjectIntHashMap;

public class Automaton2Conjunction {
	/**
	 * Returns the conjunction of automaton1 and automaton2. Does not minimise
	 * the result.
	 * 
	 * @param automaton1
	 * @param automaton2
	 */
	public static Pair<DFA, TObjectIntMap<StatePair>> conjunction(DFA automatonA, DFA automatonB,
			ProMCanceller canceller) {
		DFA conjunction = new DFAImplEdgeList();

		LinkedList<StatePair> worklist = new LinkedList<>();
		TObjectIntMap<StatePair> statePair2conjunctionState = new TObjectIntHashMap<>(10, 0.5f, -1);
		{
			StatePair statePair = new StatePair(automatonA.getInitialState(), automatonB.getInitialState());
			worklist.add(statePair);
			statePair2conjunctionState.put(statePair, conjunction.getInitialState());
		}

		EdgeIterableOutgoing itA = automatonA.getOutgoingEdgesIterator(automatonA.getInitialState());
		EdgeIterableOutgoing itB = automatonB.getOutgoingEdgesIterator(automatonB.getInitialState());
		while (!worklist.isEmpty()) {
			StatePair sourcePair = worklist.removeFirst();
			int conjunctionSourceState = statePair2conjunctionState.get(sourcePair);

			//set the state accepting if necessary
			conjunction.setAccepting(conjunctionSourceState,
					automatonA.isAccepting(sourcePair.stateA) && automatonB.isAccepting(sourcePair.stateB));

			itA.reset(sourcePair.stateA);
			itB.reset(sourcePair.stateB);

			if (itA.hasNext() && itB.hasNext()) {

				itA.nextEdge();
				itB.nextEdge();
				process(conjunction, worklist, statePair2conjunctionState, itA, itB, conjunctionSourceState);

				/**
				 * Strategy: both itA and itB are sorted on activity. Move
				 * iterators forward to obtain all equal pairs.
				 */

				while (itA.hasNext() || itB.hasNext()) {
					progress(itA, itB);
					process(conjunction, worklist, statePair2conjunctionState, itA, itB, conjunctionSourceState);
				}
			}
		}

		return Pair.of(conjunction, statePair2conjunctionState);
	}

	public static void process(DFA conjunction, LinkedList<StatePair> worklist,
			TObjectIntMap<StatePair> statePair2conjunctionState, EdgeIterableOutgoing itA, EdgeIterableOutgoing itB,
			int conjunctionSourceState) {
		short tA = itA.getActivity();
		short tB = itB.getActivity();
		if (tA == tB) {
			StatePair statePairTarget = new StatePair(itA.getTarget(), itB.getTarget());
			int conjunctionTargetState = statePair2conjunctionState.get(statePairTarget);
			if (conjunctionTargetState == statePair2conjunctionState.getNoEntryValue()) {
				//this state pair did not exist yet
				conjunctionTargetState = conjunction.addEdge(conjunctionSourceState, tA);
				statePair2conjunctionState.put(statePairTarget, conjunctionTargetState);

				worklist.add(statePairTarget);
			} else {
				//this state pair did not exist yet
				conjunction.addEdge(conjunctionSourceState, tA, conjunctionTargetState);
			}
		}
	}

	/**
	 * Progress the iterators to move closer to having the same activity.
	 * 
	 * @param itA
	 * @param itB
	 */
	private static void progress(EdgeIterableOutgoing itA, EdgeIterableOutgoing itB) {
		short tA = itA.getActivity();
		short tB = itB.getActivity();
		if (tA < tB) {
			//move tA forward
			if (itA.hasNext()) {
				itA.nextEdge();
			}
		} else if (tA > tB) {
			//move tB forward
			if (itB.hasNext()) {
				itB.nextEdge();
			}
		} else {
			//move both forward
			if (itA.hasNext()) {
				itA.nextEdge();
			}
			if (itB.hasNext()) {
				itB.nextEdge();
			}
		}
	}
}
