package org.processmining.projectedrecallandprecision.helperclasses.automata.operations;

import org.processmining.projectedrecallandprecision.helperclasses.automata.DFA;
import org.processmining.projectedrecallandprecision.helperclasses.automata.DFAImplEdgeList.EdgeIterableOutgoing;

public class StatePairEdgesIterator {

	private final EdgeIterableOutgoing itA;
	private final EdgeIterableOutgoing itB;
	private short currentAct;
	private int currentTargetA;
	private int currentTargetB;
	private short nextActA;
	private short nextActB;

	public StatePairEdgesIterator(DFA automatonA, DFA automatonB) {
		itA = automatonA.getOutgoingEdgesIterator(automatonA.getInitialState());
		itB = automatonB.getOutgoingEdgesIterator(automatonB.getInitialState());
	}

	public void reset(StatePair pair) {
		itA.reset(pair.stateA);
		itB.reset(pair.stateB);

		currentAct = -1;
		currentTargetA = -1;
		currentTargetB = -1;
		nextActA = itA.hasNext() ? itA.nextActivity() : Short.MAX_VALUE;
		nextActB = itB.hasNext() ? itB.nextActivity() : Short.MAX_VALUE;
	}

	public enum MoveType {
		onlyA, onlyB, both
	}

	public boolean hasNext() {
		return nextActA != Short.MAX_VALUE || nextActB != Short.MAX_VALUE;
	}

	public StatePairEdgesIterator.MoveType next() {
		if (nextActA < nextActB) {
			//report A
			currentAct = nextActA;
			currentTargetA = itA.getTarget();
			currentTargetB = -1;
			nextActA = itA.hasNext() ? itA.nextActivity() : Short.MAX_VALUE;
			return MoveType.onlyA;
		} else if (nextActB < nextActA) {
			//report B
			currentAct = nextActB;
			currentTargetA = -1;
			currentTargetB = itB.getTarget();
			nextActB = itB.hasNext() ? itB.nextActivity() : Short.MAX_VALUE;
			return MoveType.onlyB;
		} else {
			//report both
			currentAct = nextActA;
			currentTargetA = itA.getTarget();
			currentTargetB = itB.getTarget();
			nextActA = itA.hasNext() ? itA.nextActivity() : Short.MAX_VALUE;
			nextActB = itB.hasNext() ? itB.nextActivity() : Short.MAX_VALUE;
			return MoveType.both;
		}
	}

	public int getTargetA() {
		return currentTargetA;
	}

	public int getTargetB() {
		return currentTargetB;
	}

	public short getActivity() {
		return currentAct;
	}
}