package org.processmining.projectedrecallandprecision.helperclasses.automata.operations;

import org.processmining.projectedrecallandprecision.helperclasses.automata.DFA;

public class StatePairSet {
	public static int[] create(int stateA, int stateB) {
		return new int[] { stateA, stateB };
	}

	public static int[] add(int[] statePairSet, int stateA, int stateB) {

		//idea: keep the sources and targets sorted; first on source then on activity
		int from = binarySearch(statePairSet, stateA, stateB);
		if (from >= 0) {
			//edge already present; do nothing
			return statePairSet;
		}

		int iFrom = ~from;

		int[] result = new int[statePairSet.length + 2];
		System.arraycopy(statePairSet, 0, result, 0, iFrom);
		result[iFrom] = stateA;
		result[iFrom + 1] = stateB;
		System.arraycopy(statePairSet, iFrom, result, iFrom + 2, statePairSet.length - iFrom);

		return result;
	}

	/**
	 * Similar to Arrays.binarySearch, but works on both sources and targets
	 * 
	 * @param nodeA
	 * @param target
	 * @return
	 */
	private static int binarySearch(int[] statePairSet, int nodeA, int nodeB) {
		int low = 0;
		int high = (statePairSet.length / 2) - 1;
		int mid;
		int midVal;

		while (low <= high) {
			mid = (low + high) >>> 1;
			midVal = compare(nodeA, nodeB, statePairSet[mid * 2], statePairSet[mid * 2 + 1]);

			if (midVal < 0)
				low = mid + 1;
			else if (midVal > 0)
				high = mid - 1;
			else
				return mid * 2; // key found
		}
		return -(low * 2 + 1); // key not found.
	}

	/**
	 * Used to sort the pairs.
	 * 
	 * @param nodeA1
	 * @param nodeB1
	 * @param nodeA2
	 * @param nodeB2
	 * @return
	 */
	private static int compare(int nodeA1, int nodeB1, int nodeA2, int nodeB2) {
		if (nodeA1 < nodeA2) {
			return 1;
		} else if (nodeA1 > nodeA2) {
			return -1;
		} else if (nodeB1 < nodeB2) {
			return 1;
		} else if (nodeB1 > nodeB2) {
			return -1;
		} else {
			return 0;
		}
	}

	/**
	 * 
	 * @param automatonA
	 * @param automatonB
	 * @param statePairSet
	 * @return whether at least one pair in the set is completely accepting,
	 *         i.e. both states of the pair are accepting.
	 */
	public static boolean aPairIsBothAccepting(DFA automatonA, DFA automatonB, int[] statePairSet) {
		for (int i = 0; i < statePairSet.length; i += 2) {
			if (automatonA.isAccepting(statePairSet[i]) && automatonB.isAccepting(statePairSet[i + 1])) {
				return true;
			}
		}
		return false;
	}

	public static boolean oneStateBIsAccepting(DFA automatonB, int[] statePairSet) {
		for (int i = 0; i < statePairSet.length; i += 2) {
			if (statePairSet[i + 1] != -1 && automatonB.isAccepting(statePairSet[i + 1])) {
				return true;
			}
		}
		return false;
	}

	public static class StatePairIterator {
		private int i = 0;
		private int[] statePairSet;

		public void reset(int[] statePairSet) {
			this.statePairSet = statePairSet;
			i = -2;
		}

		public boolean hasNext() {
			return i < statePairSet.length - 2;
		}

		public void next() {
			i += 2;
		}

		public int getNodeA() {
			return statePairSet[i];
		}

		public int getNodeB() {
			return statePairSet[i + 1];
		}
	}
}
