package org.processmining.projectedrecallandprecision.helperclasses.automata;

import static org.processmining.projectedrecallandprecision.helperclasses.automata.DFAOperations.accepts;
import static org.processmining.projectedrecallandprecision.helperclasses.automata.DFAOperations.walk;
import static org.processmining.projectedrecallandprecision.helperclasses.automata.testAutomaton.check;

import java.util.ArrayList;
import java.util.List;

public class testAutomatonBasic {
	public static final List<DFA> automata;
	static {
		automata = new ArrayList<>();
		automata.add(new DFAImplEdgeList());
	}

	public static void test() throws Exception {
		for (DFA automaton : automata) {

			System.out.println("test int graph class: " + automaton.getClass());

			//test simple model
			{
				int state1 = automaton.addEdge(0, (short) 1);
				int state2 = automaton.addEdge(state1, (short) 2);
				automaton.setAccepting(state2, true);

				check(!accepts(automaton, new short[] {}), "empty trace acceptance");
				check(!accepts(automaton, new short[] { 1 }), "trace acceptance");
				check(accepts(automaton, new short[] { 1, 2 }), "trace acceptance");
				check(!accepts(automaton, new short[] { 1, 1, 2 }), "trace acceptance");

				//remove edges and states
				{
					automaton.removeOutgoingEdgesOf(state2);

					check(!automaton.isAccepting(state2), "acceptance after state removal");
					check(!accepts(automaton, new short[] {}), "empty trace acceptance after state removal");
					check(!accepts(automaton, new short[] { 1 }), "trace acceptance after state removal");
					check(!accepts(automaton, new short[] { 1, 2 }), "trace acceptance after state removal");
					check(!accepts(automaton, new short[] { 1, 1, 2 }), "trace acceptance after state removal");
					check(walk(automaton, new short[] { 1, 2 }) == state2, "walk after state removal");

					automaton.removeStateAndRedirectEdgesTo(1, 2);
					check(walk(automaton, new short[] { 1 }) == state2, "walk after state removal");
					check(walk(automaton, new short[] { 2 }) == -1, "walk after state removal");
					
					automaton.removeOutgoingEdgesOf(automaton.getInitialState());
					
					check(walk(automaton, new short[] { 1 }) == -1, "walk after state removal");
					check(walk(automaton, new short[] { 2 }) == -1, "walk after state removal");
				}
			}

			
		}
	}
}
