package org.processmining.projectedrecallandprecision.plugins;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.ProMCanceller;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.InductiveMiner.efficienttree.EfficientTree;
import org.processmining.plugins.InductiveMiner.efficienttree.ProcessTree2EfficientTree;
import org.processmining.plugins.InductiveMiner.efficienttree.UnknownTreeNodeException;
import org.processmining.plugins.InductiveMiner.plugins.dialogs.IMMiningDialog;
import org.processmining.processtree.ProcessTree;
import org.processmining.projectedrecallandprecision.framework.CompareParameters;
import org.processmining.projectedrecallandprecision.framework.CompareParameters.RecallName;
import org.processmining.projectedrecallandprecision.helperclasses.AutomatonFailedException;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult.ProjectedMeasuresFailedException;

public class CompareLog2ProcessTreePlugin {

	@Plugin(name = "Compute projected fitness and precision (log and process tree)", returnLabels = {
			"fitness and precision" }, returnTypes = {
					ProjectedRecallPrecisionResult.class }, parameterLabels = { "Log", "Model" }, userAccessible = true)
	@UITopiaVariant(affiliation = IMMiningDialog.affiliation, author = IMMiningDialog.author, email = IMMiningDialog.email)
	@PluginVariant(variantLabel = "Mine a Process Tree, dialog", requiredParameterLabels = { 0, 1 })
	public ProjectedRecallPrecisionResult measure(final UIPluginContext context, XLog log, ProcessTree model)
			throws AutomatonFailedException, InterruptedException, UnknownTreeNodeException,
			ProjectedMeasuresFailedException {
		CompareParametersDialog dialog = new CompareParametersDialog(log,
				ProcessTree2EfficientTree.convert(model).getInt2activity().length, RecallName.fitness);
		InteractionResult dialogResult = context.showWizard("Projected Conformance Checking", true, true, dialog);
		if (dialogResult != InteractionResult.FINISHED) {
			context.getFutureResult(0).cancel(false);
			return null;
		}

		context.log("Comparing...");

		ProjectedRecallPrecisionResult result = measure(log, model, dialog.getMiningParameters(), new ProMCanceller() {
			public boolean isCancelled() {
				return context.getProgress().isCancelled();
			}
		});

		if (result == null || context.getProgress().isCancelled()) {
			context.getFutureResult(0).cancel(false);
			return null;
		}

		return result;
	}

	@Deprecated
	/**
	 * Preferably use a variant with a canceller.
	 * 
	 * @param log
	 * @param model
	 * @param parameters
	 * @param canceller
	 * @return
	 * @throws AutomatonFailedException
	 * @throws InterruptedException
	 * @throws UnknownTreeNodeException
	 * @throws ProjectedMeasuresFailedException
	 */
	public static ProjectedRecallPrecisionResult measure(XLog log, ProcessTree model, CompareParameters parameters)
			throws AutomatonFailedException, InterruptedException, UnknownTreeNodeException,
			ProjectedMeasuresFailedException {
		return measure(log, model, parameters, new ProMCanceller() {
			public boolean isCancelled() {
				return false;
			}
		});
	}

	public static ProjectedRecallPrecisionResult measure(XLog log, ProcessTree model, CompareParameters parameters,
			ProMCanceller canceller) throws AutomatonFailedException, InterruptedException, UnknownTreeNodeException,
			ProjectedMeasuresFailedException {
		EfficientTree efficientTree = ProcessTree2EfficientTree.convert(model);
		return CompareLog2EfficientTreePlugin.measure(log, efficientTree, parameters, canceller);
	}

}
