package org.processmining.projectedrecallandprecision.result;

import javax.swing.JComponent;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.contexts.util.HtmlPanel;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.InductiveMiner.plugins.dialogs.IMMiningDialog;
import org.processmining.plugins.graphviz.colourMaps.ColourMap;
import org.processmining.plugins.graphviz.colourMaps.ColourMapWhiteRed;
import org.processmining.projectedrecallandprecision.result.ProjectedRecallPrecisionResult.ProjectedMeasuresFailedException;

public class ProjectedRecallPrecisionVisualiserTable {
	@Plugin(level = PluginLevel.Regular, name = "Projected recall/precision table", returnLabels = {
			"Projected recall/precision table" }, returnTypes = {
					JComponent.class }, parameterLabels = { "Projected recall/precision" }, userAccessible = true)
	@Visualizer
	@UITopiaVariant(affiliation = IMMiningDialog.affiliation, author = IMMiningDialog.author, email = IMMiningDialog.email)
	@PluginVariant(variantLabel = "Show projected recall and precision", requiredParameterLabels = { 0 })
	public JComponent visualize(PluginContext context, ProjectedRecallPrecisionResult recallPrecisionResult)
			throws ProjectedMeasuresFailedException {
		boolean extended = false;
		return new HtmlPanel(generateHtml(recallPrecisionResult, extended));
	}

	@Plugin(level = PluginLevel.Regular, name = "Projected recall/precision table (extended)", returnLabels = {
			"Projected recall/precision table" }, returnTypes = {
					JComponent.class }, parameterLabels = { "Projected recall/precision" }, userAccessible = true)
	@Visualizer
	@UITopiaVariant(affiliation = IMMiningDialog.affiliation, author = IMMiningDialog.author, email = IMMiningDialog.email)
	@PluginVariant(variantLabel = "Show projected recall and precision", requiredParameterLabels = { 0 })
	public JComponent visualizeExtended(PluginContext context, ProjectedRecallPrecisionResult recallPrecisionResult)
			throws ProjectedMeasuresFailedException {
		boolean extended = true;
		return new HtmlPanel(generateHtml(recallPrecisionResult, extended));
	}

	public static String generateHtml(ProjectedRecallPrecisionResult recallPrecisionResult, boolean extended)
			throws ProjectedMeasuresFailedException {
		StringBuilder text = new StringBuilder();
		text.append("<html>");
		if (recallPrecisionResult.isRecallComputed()) {
			text.append(recallPrecisionResult.getRecallName() + ": " + recallPrecisionResult.getRecall());
			text.append("<br>");
		}
		if (recallPrecisionResult.isPrecisionComputed()) {
			text.append("precision: " + recallPrecisionResult.getPrecision());
			text.append("<br>");
		}

		if (extended) {
			ColourMap colourMap = new ColourMapWhiteRed();

			//recall by activity
			if (recallPrecisionResult.getK() >= 1 && recallPrecisionResult.isRecallComputed()) {
				recallActivity(recallPrecisionResult, text, colourMap);
			}

			//recall by pair
			if (recallPrecisionResult.getK() >= 2 && recallPrecisionResult.isRecallComputed()) {
				recallPair(recallPrecisionResult, text, colourMap);
			}

			//precision by activity
			if (recallPrecisionResult.getK() >= 1 && recallPrecisionResult.isPrecisionComputed()) {
				precisionActivity(recallPrecisionResult, text, colourMap);
			}

			//precision by pair
			if (recallPrecisionResult.getK() >= 2 && recallPrecisionResult.isPrecisionComputed()) {
				precisionPair(recallPrecisionResult, text, colourMap);
			}
		}

		text.append("</html>");
		return text.toString();
	}

	public static void recallActivity(ProjectedRecallPrecisionResult recallPrecisionResult, StringBuilder text,
			ColourMap colourMap) throws ProjectedMeasuresFailedException {
		text.append(recallPrecisionResult.getRecallName() + " by activity");
		text.append("<br>");
		text.append("<table>");
		for (String activity : recallPrecisionResult.getActivities()) {
			text.append("<tr>");
			text.append("<td>" + activity + "</td>");
			double r = recallPrecisionResult.getRecall(activity);
			if (r >= -0.01) {
				text.append(
						"<td bgcolor=\"" + ColourMap.toHexString(colourMap.colour((long) (r * 1000), 1000)) + "\">");
				text.append(String.format("%.3f", r));
				text.append("</td>");
			} else {
				text.append("<td></td>");
			}
			text.append("</tr>");
		}
		text.append("</table>");
		text.append("<br>");
	}

	public static void recallPair(ProjectedRecallPrecisionResult recallPrecisionResult, StringBuilder text,
			ColourMap colourMap) throws ProjectedMeasuresFailedException {
		text.append(recallPrecisionResult.getRecallName() + " by pair");
		text.append("<br>");
		text.append("<table>");
		//header
		{
			text.append("<tr>");
			text.append("<td></td>");
			for (String activity : recallPrecisionResult.getActivities()) {
				text.append("<td>" + activity + "</td>");
			}
			text.append("</tr>");
		}
		//rows
		for (String activity1 : recallPrecisionResult.getActivities()) {
			text.append("<tr>");
			text.append("<td>" + activity1 + "</td>");
			for (String activity2 : recallPrecisionResult.getActivities()) {
				if (recallPrecisionResult.comparator.compare(activity1, activity2) > 0) {
					double r = recallPrecisionResult.getRecall(activity1, activity2);
					if (r > -0.01) {
						text.append("<td bgcolor=\"" + ColourMap.toHexString(colourMap.colour((long) (r * 1000), 1000))
								+ "\">");
						text.append(String.format("%.3f", r));
					} else {
						text.append("<td>");
					}
				} else {
					text.append("<td>");
				}
				text.append("</td>");
			}
			text.append("</tr>");
		}
		text.append("</table>");
		text.append("<br>");
	}

	public static void precisionPair(ProjectedRecallPrecisionResult recallPrecisionResult, StringBuilder text,
			ColourMap colourMap) throws ProjectedMeasuresFailedException {
		text.append("precision by pair");
		text.append("<br>");
		text.append("<table>");
		//header
		{
			text.append("<tr>");
			text.append("<td></td>");
			for (String activity : recallPrecisionResult.getActivities()) {
				text.append("<td>" + activity + "</td>");
			}
			text.append("</tr>");
		}
		//rows
		for (String activity1 : recallPrecisionResult.getActivities()) {
			text.append("<tr>");
			text.append("<td>" + activity1 + "</td>");
			for (String activity2 : recallPrecisionResult.getActivities()) {
				if (recallPrecisionResult.comparator.compare(activity1, activity2) > 0) {
					double r = recallPrecisionResult.getPrecision(activity1, activity2);
					if (r > -0.01) {
						text.append("<td bgcolor=\"" + ColourMap.toHexString(colourMap.colour((long) (r * 1000), 1000))
								+ "\">");
						text.append(String.format("%.3f", r));
					} else {
						text.append("<td>");
					}
				} else {
					text.append("<td>");
				}
				text.append("</td>");
			}
			text.append("</tr>");
		}
		text.append("</table>");
	}

	public static void precisionActivity(ProjectedRecallPrecisionResult recallPrecisionResult, StringBuilder text,
			ColourMap colourMap) throws ProjectedMeasuresFailedException {
		text.append("precision by activity");
		text.append("<br>");
		text.append("<table>");
		for (String activity : recallPrecisionResult.getActivities()) {
			text.append("<tr>");
			text.append("<td>" + activity + "</td>");
			double r = recallPrecisionResult.getPrecision(activity);
			if (r >= -0.01) {
				text.append(
						"<td bgcolor=\"" + ColourMap.toHexString(colourMap.colour((long) (r * 1000), 1000)) + "\">");
				text.append(String.format("%.3f", r));
				text.append("</td>");
			} else {
				text.append("<td></td>");
			}
			text.append("</tr>");
		}
		text.append("</table>");
		text.append("<br>");
	}
}
